@@ -1,11 +1,14 @@
 #pragma once
-#include <avnd/concepts/field_names.hpp>
 #include <avnd/binding/ossia/ossia_to_curve.hpp>
 #include <avnd/binding/ossia/value.hpp>
+#include <avnd/concepts/field_names.hpp>
 #include <avnd/concepts/generic.hpp>
 #include <avnd/concepts/parameter.hpp>
 #include <avnd/introspection/range.hpp>
+#include <avnd/introspection/type_wrapper.hpp>
+#include <avnd/introspection/vecf.hpp>
 #include <boost/mp11/algorithm.hpp>
+#include <ossia/network/value/value.hpp>
 #include <ossia/network/value/value_conversion.hpp>
 
 #include <type_traits>
@@ -143,7 +146,7 @@ template <typename T>
 using is_bool = std::is_same<bool, T>;
 template <typename T>
 using is_variant
-    = std::integral_constant<bool, avnd::variant_ish<T> || oscr::type_wrapper<T>>;
+    = std::integral_constant<bool, avnd::variant_ish<T> || avnd::type_wrapper<T>>;
 
 // Exclude string-ish stuff
 template <typename T>
@@ -340,7 +343,7 @@ struct from_ossia_value_impl
   operator()(const ossia::value& src, F& dst)
   {
     constexpr int sz = boost::pfr::tuple_size_v<F>;
-    if constexpr(vecf_compatible<F>())
+    if constexpr(avnd::vecf_compatible<F>())
     {
       if constexpr(sz == 2)
       {
@@ -413,6 +416,50 @@ struct from_ossia_value_impl
     return true;
   }
 
+  template <typename T1, typename T2>
+  bool operator()(const ossia::value& src, std::pair<T1, T2>& f)
+  {
+    if(auto vec = src.target<std::vector<ossia::value>>(); vec && vec->size() >= 2)
+    {
+      (*this)((*vec)[0], f.first);
+      (*this)((*vec)[1], f.second);
+      return true;
+    }
+    else
+    {
+      if(auto vec = ossia::convert<std::vector<ossia::value>>(src); vec.size() >= 2)
+      {
+        (*this)(vec[0], f.first);
+        (*this)(vec[1], f.second);
+      }
+      return false;
+    }
+  }
+
+  template <typename... T>
+  bool operator()(const ossia::value& src, std::tuple<T...>& f)
+  {
+    if(auto vec = src.target<std::vector<ossia::value>>();
+       vec && vec->size() >= sizeof...(T))
+    {
+      [this, vec, &f]<std::size_t... I>(std::index_sequence<I...>) {
+        ((*this)((*vec)[I], std::get<I>(f)), ...);
+      }(std::make_index_sequence<sizeof...(T)>{});
+      return true;
+    }
+    else
+    {
+      if(auto vec = ossia::convert<std::vector<ossia::value>>(src); vec.size() >= 2)
+      {
+        [this, vec, &f]<std::size_t... I>(std::index_sequence<I...>) {
+          ((*this)(vec[I], std::get<I>(f)), ...);
+        }(std::make_index_sequence<sizeof...(T)>{});
+        return true;
+      }
+      return false;
+    }
+  }
+
   template <
       template <typename> typename Pred, typename Internal,
       template <typename...> typename T, typename... Args>
@@ -623,7 +670,7 @@ struct from_ossia_value_impl
     */
   }
 
-  bool operator()(const ossia::value& src, oscr::type_wrapper auto& f)
+  bool operator()(const ossia::value& src, avnd::type_wrapper auto& f)
   {
     auto& [obj] = f;
     return (*this)(src, obj);
@@ -790,9 +837,77 @@ struct from_ossia_value_impl
     return false;
   }
 
+  template <avnd::list_ish T>
+    requires(!avnd::string_ish<T> && !avnd::vector_ish<T>)
+  bool operator()(const ossia::value& src, T& f)
+  {
+    switch(src.get_type())
+    {
+      case ossia::val_type::VEC2F: {
+        if constexpr(float_compatible<typename T::value_type>)
+        {
+          ossia::vec2f v = *src.target<ossia::vec2f>();
+          f.clear();
+          for(auto& val : v)
+            f.push_back(val);
+          return true;
+        }
+        else
+        {
+          return false;
+        }
+        break;
+      }
+      case ossia::val_type::VEC3F: {
+        if constexpr(float_compatible<typename T::value_type>)
+        {
+          ossia::vec3f v = *src.target<ossia::vec3f>();
+          f.clear();
+          for(auto& val : v)
+            f.push_back(val);
+          return true;
+        }
+        else
+        {
+          return false;
+        }
+        break;
+      }
+      case ossia::val_type::VEC4F: {
+        if constexpr(float_compatible<typename T::value_type>)
+        {
+          ossia::vec4f v = *src.target<ossia::vec4f>();
+          f.clear();
+          for(auto& val : v)
+            f.push_back(val);
+          return true;
+        }
+        else
+        {
+          return false;
+        }
+        break;
+      }
+      case ossia::val_type::LIST: {
+        const std::vector<ossia::value>& vl = *src.target<std::vector<ossia::value>>();
+        f.clear();
+        for(auto& val : vl)
+        {
+          typename T::value_type res;
+          (*this)(val, res);
+          f.push_back(res);
+        }
+        return true;
+        break;
+      }
+      default:
+        return false;
+        break;
+    }
+  }
   template <avnd::vector_ish T>
-    requires(!avnd::string_ish<T>) bool
-  operator()(const ossia::value& src, T& f)
+    requires(!avnd::string_ish<T>)
+  bool operator()(const ossia::value& src, T& f)
   {
     switch(src.get_type())
     {
@@ -1130,10 +1245,11 @@ bool from_ossia_value_rec(const ossia::value& src, T& dst)
 }
 
 template <typename T>
+  requires std::is_aggregate_v<T>
 bool from_ossia_value(const ossia::value& src, T& dst)
 {
   using type = std::decay_t<T>;
-  static_assert(!oscr::type_wrapper<T>);
+  static_assert(!avnd::type_wrapper<T>);
   constexpr int sz = boost::pfr::tuple_size_v<T>;
   if constexpr(sz == 0)
   {
@@ -1141,7 +1257,7 @@ bool from_ossia_value(const ossia::value& src, T& dst)
     // FIXME unless we have an optional parameter !
     return true;
   }
-  else if constexpr(vecf_compatible<type>())
+  else if constexpr(avnd::vecf_compatible<type>())
   {
     if constexpr(sz == 2)
     {
@@ -1203,12 +1319,6 @@ inline bool from_ossia_value(const ossia::value& src, const char*& dst)
     return false;
   }
 }
-template <avnd::vector_ish T>
-  requires(!avnd::string_ish<T>) bool
-from_ossia_value(const ossia::value& src, T& dst)
-{
-  return from_ossia_value_impl{}(src, dst);
-}
 
 template <
     template <typename, std::size_t, typename...> typename T, typename Val,
@@ -1226,30 +1336,25 @@ from_ossia_value(const ossia::value& src, T<N>& dst)
   return from_ossia_value_impl{}(src, dst);
 }
 
-template <avnd::variant_ish T>
+template <typename T>
+  requires(
+      avnd::variant_ish<T> || avnd::set_ish<T> || avnd::map_ish<T> || avnd::bitset_ish<T>
+      || avnd::pair_ish<T> || avnd::tuple_ish<T> || avnd::iterable_ish<T>
+      || (avnd::vector_ish<T> && !avnd::string_ish<T>))
 bool from_ossia_value(const ossia::value& src, T& dst)
 {
   return from_ossia_value_impl{}(src, dst);
 }
-
-template <avnd::set_ish T>
-bool from_ossia_value(const ossia::value& src, T& dst)
+inline bool from_ossia_value(const ossia::value& src, std::vector<ossia::value>& dst)
 {
-  return from_ossia_value_impl{}(src, dst);
+  dst = ossia::convert<std::vector<ossia::value>>(src);
+  return true;
 }
-
-template <avnd::map_ish T>
-bool from_ossia_value(const ossia::value& src, T& dst)
+inline bool from_ossia_value(const ossia::value& src, ossia::value_map_type& dst)
 {
-  return from_ossia_value_impl{}(src, dst);
+  dst = ossia::convert<ossia::value_map_type>(src);
+  return true;
 }
-
-template <avnd::bitset_ish T>
-bool from_ossia_value(const ossia::value& src, T& dst)
-{
-  return from_ossia_value_impl{}(src, dst);
-}
-
 template <avnd::optional_ish T>
 void from_ossia_value(const ossia::value& src, T& dst)
 {
@@ -1266,7 +1371,7 @@ void from_ossia_value(const ossia::value& src, T& dst)
   }
 }
 
-template <oscr::type_wrapper T>
+template <avnd::type_wrapper T>
 bool from_ossia_value(const ossia::value& src, T& dst)
 {
   auto& [obj] = dst;
@@ -1293,6 +1398,12 @@ inline void from_ossia_value(auto& field, const ossia::value& src, ossia::value&
   dst = src;
 }
 
+inline void
+from_ossia_value(auto& field, const ossia::value& src, std::optional<ossia::value>& dst)
+{
+  dst = src;
+}
+
 inline void from_ossia_value(auto& field, const ossia::value& src, auto& dst)
 {
   from_ossia_value(src, dst);
