@@ -1,6 +1,11 @@
 #pragma once
+#include <avnd/binding/ossia/builtin_ports.hpp>
+#include <avnd/binding/ossia/callbacks.hpp>
 #include <avnd/binding/ossia/configure.hpp>
+#include <avnd/binding/ossia/controls.hpp>
+#include <avnd/binding/ossia/dynamic_ports.hpp>
 #include <avnd/binding/ossia/ffts.hpp>
+#include <avnd/binding/ossia/message_process.hpp>
 #include <avnd/binding/ossia/port_run_postprocess.hpp>
 #include <avnd/binding/ossia/port_run_preprocess.hpp>
 #include <avnd/binding/ossia/port_setup.hpp>
@@ -23,7 +28,6 @@
 #include <avnd/wrappers/process_adapter.hpp>
 #include <avnd/wrappers/smooth.hpp>
 #include <avnd/wrappers/widgets.hpp>
-#include <boost/smart_ptr/atomic_shared_ptr.hpp>
 #include <ossia/dataflow/audio_port.hpp>
 #include <ossia/dataflow/graph_node.hpp>
 #include <ossia/dataflow/node_process.hpp>
@@ -96,187 +100,6 @@ static void safety_checks(const ossia::token_request& tk, ossia::exec_state_faca
 
 // void process(double** in, double** out, int N)
 
-template <typename T>
-struct builtin_arg_audio_ports
-{
-  void init(ossia::inlets& inlets, ossia::outlets& outlets) { }
-};
-
-template <avnd::audio_argument_processor T>
-struct builtin_arg_audio_ports<T>
-{
-  ossia::audio_inlet in;
-  ossia::audio_outlet out;
-
-  void init(ossia::inlets& inlets, ossia::outlets& outlets)
-  {
-    inlets.push_back(&in);
-    outlets.push_back(&out);
-  }
-};
-
-template <typename T>
-struct builtin_message_value_ports
-{
-  void init(ossia::inlets& inlets) { }
-};
-
-template <typename T>
-  requires(avnd::messages_introspection<T>::size > 0)
-struct builtin_message_value_ports<T>
-{
-  ossia::value_inlet message_inlets[avnd::messages_introspection<T>::size];
-  void init(ossia::inlets& inlets)
-  {
-    for(auto& in : message_inlets)
-    {
-      inlets.push_back(&in);
-    }
-  }
-};
-
-template <typename Field>
-using controls_type = std::decay_t<decltype(Field::value)>;
-
-template <typename T>
-using atomic_shared_ptr = boost::atomic_shared_ptr<T>;
-
-template <typename T>
-struct controls_mirror
-{
-  static constexpr int i_size = avnd::control_input_introspection<T>::size;
-  static constexpr int o_size = avnd::control_output_introspection<T>::size;
-  using i_tuple
-      = avnd::filter_and_apply<controls_type, avnd::control_input_introspection, T>;
-  using o_tuple
-      = avnd::filter_and_apply<controls_type, avnd::control_output_introspection, T>;
-
-  controls_mirror()
-  {
-    inputs.load(new i_tuple);
-    outputs.load(new o_tuple);
-  }
-
-  [[no_unique_address]] atomic_shared_ptr<i_tuple> inputs;
-  [[no_unique_address]] atomic_shared_ptr<o_tuple> outputs;
-
-  std::bitset<i_size> inputs_bits;
-  std::bitset<o_size> outputs_bits;
-};
-
-template <typename T>
-struct controls_input_queue
-{
-};
-template <typename T>
-struct controls_output_queue
-{
-};
-template <typename T>
-  requires(avnd::control_input_introspection<T>::size > 0)
-struct controls_input_queue<T>
-{
-  static constexpr int i_size = avnd::control_input_introspection<T>::size;
-  using i_tuple
-      = avnd::filter_and_apply<controls_type, avnd::control_input_introspection, T>;
-
-  moodycamel::ConcurrentQueue<i_tuple> ins_queue;
-  std::bitset<i_size> inputs_set;
-};
-
-template <typename T>
-  requires(avnd::control_output_introspection<T>::size > 0)
-struct controls_output_queue<T>
-{
-  static constexpr int o_size = avnd::control_output_introspection<T>::size;
-  using o_tuple
-      = avnd::filter_and_apply<controls_type, avnd::control_output_introspection, T>;
-
-  moodycamel::ConcurrentQueue<o_tuple> outs_queue;
-
-  std::bitset<o_size> outputs_set;
-};
-
-template <typename T>
-struct controls_queue
-    : controls_input_queue<T>
-    , controls_output_queue<T>
-{
-};
-
-template <typename Self, std::size_t Idx, typename Field, typename R, typename... Args>
-struct do_callback;
-
-template <typename Self, std::size_t Idx, typename Field, typename R, typename... Args>
-  requires(!requires { Field::timestamp; })
-struct do_callback<Self, Idx, Field, R, Args...>
-{
-  Self& self;
-  Field& field;
-  R operator()(Args... args)
-  {
-    // Idx is the index of the port in the complete input array.
-    // We need to map it to the callback index.
-    ossia::value_outlet& port = tuplet::get<Idx>(self.ossia_outlets.ports);
-    if constexpr(sizeof...(Args) == 0)
-    {
-      port.data.write_value(ossia::impulse{}, 0);
-    }
-    else if constexpr(sizeof...(Args) == 1)
-    {
-      port.data.write_value(to_ossia_value(field, args...), 0);
-    }
-    else
-    {
-      std::vector<ossia::value> vec{to_ossia_value(field, args)...};
-      port.data.write_value(std::move(vec), 0);
-    }
-
-    if constexpr(!std::is_void_v<R>)
-      return R{};
-  }
-};
-
-template <typename Self, std::size_t Idx, typename Field, typename R, typename... Args>
-  requires(requires { Field::timestamp; })
-struct do_callback<Self, Idx, Field, R, Args...>
-{
-  Self& self;
-  Field& field;
-
-  void do_call(int64_t ts)
-  {
-    // Idx is the index of the port in the complete input array.
-    // We need to map it to the callback index.
-    ossia::value_outlet& port = tuplet::get<Idx>(self.ossia_outlets.ports);
-    port.data.write_value(ossia::impulse{}, ts);
-  }
-
-  template <typename U>
-  void do_call(int64_t ts, U&& u)
-  {
-    // Idx is the index of the port in the complete input array.
-    // We need to map it to the callback index.
-    ossia::value_outlet& port = tuplet::get<Idx>(self.ossia_outlets.ports);
-    port.data.write_value(to_ossia_value(field, std::forward<U>(u)), ts);
-  }
-
-  template <typename... U>
-  void do_call(int64_t ts, U&&... us)
-  {
-    ossia::value_outlet& port = tuplet::get<Idx>(self.ossia_outlets.ports);
-    std::vector<ossia::value> vec{to_ossia_value(field, us)...};
-    port.data.write_value(std::move(vec), ts);
-  }
-
-  R operator()(Args... args)
-  {
-    do_call(static_cast<Args&&>(args)...);
-    if constexpr(!std::is_void_v<R>)
-      return R{};
-  }
-};
-
 template <typename T>
 class safe_node_base_base : public ossia::nonowning_graph_node
 {
@@ -286,6 +109,9 @@ public:
   {
   }
 
+  int start_frame_for_this_tick = 0;
+  int frame_count_for_this_tick = 0;
+
   int buffer_size{};
   double sample_rate{};
   double tempo{ossia::root_tempo};
@@ -294,6 +120,8 @@ public:
 
   [[no_unique_address]] oscr::builtin_arg_audio_ports<T> audio_ports;
 
+  [[no_unique_address]] oscr::builtin_arg_value_ports<T> arg_value_ports;
+
   [[no_unique_address]] oscr::builtin_message_value_ports<T> message_ports;
 
   [[no_unique_address]] oscr::inlet_storage<T> ossia_inlets;
@@ -324,10 +152,14 @@ public:
 
   [[no_unique_address]] controls_queue<T> control;
 
+  [[no_unique_address]] oscr::dynamic_ports_storage<T> dynamic_ports;
+
+  [[no_unique_address]] oscr::message_processor<T> messages;
+
   using control_input_values_type
-      = avnd::filter_and_apply<controls_type, avnd::control_input_introspection, T>;
+      = avnd::filter_and_apply<controls_type_t, avnd::control_input_introspection, T>;
   using control_output_values_type
-      = avnd::filter_and_apply<controls_type, avnd::control_output_introspection, T>;
+      = avnd::filter_and_apply<controls_type_t, avnd::control_output_introspection, T>;
 };
 
 template <std::size_t Index, typename F>
@@ -348,9 +180,6 @@ public:
   using midi_in_info = avnd::midi_input_introspection<T>;
   using midi_out_info = avnd::midi_output_introspection<T>;
 
-  static constexpr int total_input_ports = avnd::total_input_count<T>();
-  static constexpr int total_output_ports = avnd::total_output_count<T>();
-
   uint64_t instance{};
 
   safe_node_base(int buffer_size, double sample_rate, uint64_t uid)
@@ -360,10 +189,8 @@ public:
     this->buffer_size = buffer_size;
     this->sample_rate = sample_rate;
 
-    this->m_inlets.reserve(total_input_ports + 1);
-    this->m_outlets.reserve(total_output_ports + 1);
-
     this->audio_ports.init(this->m_inlets, this->m_outlets);
+    this->arg_value_ports.init(this->m_inlets, this->m_outlets);
     this->message_ports.init(this->m_inlets);
     this->soundfiles.init(this->impl);
 
@@ -409,8 +236,9 @@ public:
   template <typename Functor, typename... Args>
   void process_all_ports(Args&&... args)
   {
-    for(auto& [impl, i, o] : this->impl.full_state())
+    for(auto [impl, i, o] : this->impl.full_state())
     {
+      static_assert(std::is_reference_v<decltype(impl)>);
       Functor f{*this, impl, args...};
       if constexpr(avnd::inputs_type<T>::size > 0)
         process_inputs_impl(f, i);
@@ -456,6 +284,7 @@ public:
 
     // Small setup step for the variable channel counts
     this->process_all_ports<setup_variable_audio_ports<safe_node_base, T>>();
+    this->process_all_ports<setup_raw_ossia_ports<safe_node_base, T>>();
   }
 
   void finish_init()
@@ -514,7 +343,39 @@ public:
     this->control.inputs_set.set(N);
   }
 
-  void audio_configuration_changed()
+  template <typename Val, std::size_t N>
+  void control_updated_from_ui(Val&& new_value, int dynamic_port)
+  {
+    // FIXME how to combine dynamic_ports and control_input
+    if constexpr(requires { avnd::effect_container<T>::multi_instance; })
+    {
+      for(const auto& state : this->impl.full_state())
+      {
+        // Replace the value in the field
+        auto& field = avnd::dynamic_ports_input_introspection<T>::template field<N>(
+            state.inputs);
+
+        // OPTIMIZEME we're loosing a few allocations here that should be gc'd
+        field.ports[dynamic_port].value = new_value;
+
+        if_possible(field.update(state.effect));
+      }
+    }
+    else
+    {
+      // Replace the value in the field
+      auto& field = avnd::dynamic_ports_input_introspection<T>::template field<N>(
+          this->impl.inputs());
+
+      std::swap(field.ports[dynamic_port].value, new_value);
+
+      if_possible(field.update(this->impl.effect));
+    }
+
+    // Mark the control as changed
+    this->control.inputs_set.set(N);
+  }
+  void audio_configuration_changed(ossia::exec_state_facade st)
   {
     // qDebug() << "New Audio configuration: "
     //          << this->channels.actual_runtime_inputs
@@ -525,7 +386,10 @@ public:
         .output_channels = this->channels.actual_runtime_outputs,
         .frames_per_buffer = this->buffer_size,
         .rate = this->sample_rate,
-        .instance = this->instance};
+        .instance = this->instance,
+        .samples_to_model = st.samplesToModel(),
+        .model_to_samples = st.modelToSamples(),
+    };
 
     // This allocates the buffers that may be used for conversion
     // if e.g. we have an API that works with doubles,
@@ -583,7 +447,8 @@ public:
     }
   }
 
-  bool prepare_run(const ossia::token_request& tk, int start, int frames)
+  bool prepare_run(
+      const ossia::token_request& tk, ossia::exec_state_facade st, int start, int frames)
   {
     // Update metadatas
     this->update_tempo(tk.tempo);
@@ -599,7 +464,7 @@ public:
 
     if(changed)
     {
-      audio_configuration_changed();
+      audio_configuration_changed(st);
     }
 
     // Clean up MIDI output ports
@@ -608,184 +473,49 @@ public:
     // Clean up sample-accurate control output ports
     this->control_buffers.clear_outputs(this->impl);
 
+    // Save the current start and frames
+    this->start_frame_for_this_tick = start;
+    this->frame_count_for_this_tick = frames;
+
     // Process inputs of all sorts
     this->process_all_ports<process_before_run<safe_node_base, T>>(start, frames);
 
     // Process messages
     if constexpr(avnd::messages_type<T>::size > 0)
-      process_messages();
+    {
+      avnd::messages_introspection<T>::for_all(
+          [&](auto m) { this->messages(*this, m); });
+    }
 
     return true;
   }
 
-  template <auto Idx, typename M>
-  void invoke_message(const ossia::value& val, avnd::field_reflection<Idx, M>)
-  {
-    if constexpr(!std::is_void_v<avnd::message_reflection<M>>)
-    {
-      using refl = avnd::message_reflection<M>;
-      constexpr auto arg_count = refl::count;
-      constexpr auto f = avnd::message_get_func<M>();
-
-      if constexpr(arg_count == 0)
-      {
-        for(auto& m : this->impl.effects())
-        {
-          if constexpr(std::is_member_function_pointer_v<decltype(f)>)
-          {
-            if constexpr(requires { M{}(); })
-              M{}();
-            else
-              (m.*f)();
-          }
-          else
-            f();
-        }
-      }
-      else if constexpr(arg_count == 1)
-      {
-        if constexpr(std::is_same_v<avnd::first_message_argument<M>, T&>)
-        {
-          for(auto& m : this->impl.effects())
-          {
-            if constexpr(std::is_member_function_pointer_v<decltype(f)>)
-            {
-              if constexpr(requires { M{}(m); })
-                M{}(m);
-              else if constexpr(requires { (m.*f)(m); })
-                (m.*f)(m);
-            }
-            else
-              f(m);
-          }
-        }
-        else
-        {
-          constexpr M field;
-
-          using arg_type = std::decay_t<avnd::first_message_argument<M>>;
-          arg_type arg;
-          from_ossia_value(field, val, arg, avnd::field_index<Idx>{});
-
-          for(auto& m : this->impl.effects())
-          {
-            if constexpr(std::is_member_function_pointer_v<decltype(f)>)
-            {
-              if constexpr(requires { M{}(arg); })
-                M{}(arg);
-              else if constexpr(requires { (m.*f)(arg); })
-                (m.*f)(arg);
-            }
-            else
-            {
-              f(arg);
-            }
-          }
-          /*
-          for (auto& m : this->impl.effects())
-          {
-            if constexpr (std::is_member_function_pointer_v<decltype(f)>)
-            {
-              if constexpr(requires { M{}(m); })
-                M{}(m);
-              else if constexpr(requires { (m.*f)(m); })
-                (m.*f)(m);
-            }
-            else
-              f(m);
-          }*/
-          //   FIXME ! oscquery_mapper.hpp already has all this code
-          // using first_arg = boost::mp11::mp_first<typename refl::arguments>;
-          // const auto& v = ossia::convert<std::decay_t<first_arg>>(val);
-          // for(auto& m : this->impl.effects())
-          // {
-          //   if constexpr (std::is_member_function_pointer_v<decltype(f)>)
-          //     (m.*f)(v);
-          //   else
-          //     f(v);
-          // }
-        }
-      }
-      else
-      {
-        if constexpr(arg_count == 2)
-        {
-          constexpr M field;
-          using arg_type = std::decay_t<avnd::second_message_argument<M>>;
-          arg_type arg;
-          from_ossia_value(field, val, arg, avnd::field_index<Idx>{});
-          for(auto& m : this->impl.effects())
-          {
-            if constexpr(std::is_member_function_pointer_v<decltype(f)>)
-            {
-              if constexpr(requires { M{}(m, arg); })
-                M{}(m, arg);
-              else if constexpr(requires { (m.*f)(m, arg); })
-                (m.*f)(m, arg);
-            }
-            else
-              f(m, arg);
-          }
-        }
-        // TODO use vecf, list, etc...
-        // FIXME
-      }
-    }
-    else
-    {
-      //FIXME > 1 arguments
-    }
-  }
-
-  template <auto Idx, typename M>
-  void process_message(avnd::field_reflection<Idx, M>)
-  {
-    ossia::value_inlet& inl = this->message_ports.message_inlets[Idx];
-    if(inl.data.get_data().empty())
-      return;
-    for(const auto& val : inl.data.get_data())
-    {
-      invoke_message(val.value, avnd::field_reflection<Idx, M>{});
-    }
-  }
-
-  void process_messages()
-  {
-    avnd::messages_introspection<T>::for_all([&](auto m) { process_message(m); });
-  }
-
   void process_smooth() { this->smooth.smooth_all(this->impl); }
 
-  auto make_controls_in_tuple()
+  typename controls_input_queue<T>::i_tuple make_controls_in_tuple()
   {
     // We only care about the inputs of the first one, since they're all the same
-    for(auto& state : this->impl.full_state())
+    for(auto state : this->impl.full_state())
     {
       return avnd::control_input_introspection<T>::filter_tuple(
           state.inputs, [](auto& field) { return field.value; });
     }
+    return {};
   }
-  auto make_controls_out_tuple()
+
+  typename controls_output_queue<T>::o_tuple make_controls_out_tuple()
   {
     // Note that this does not yet make a lot of sens for polyphonic effects
-    for(auto& state : this->impl.full_state())
+    for(auto state : this->impl.full_state())
     {
       return avnd::control_output_introspection<T>::filter_tuple(
           state.outputs, [](auto& field) { return field.value; });
     }
+    return {};
   }
 
   void finish_run()
   {
-    // Copy output events
-    this->process_all_ports<process_after_run<safe_node_base, T>>();
-
-    // Clean up MIDI inputs
-    this->midi_buffers.clear_inputs(this->impl);
-
-    // Clean up sample-accurate control input ports
-    this->control_buffers.clear_inputs(this->impl);
-
     // Clear control bitsets for UI
     if constexpr(avnd::control_input_introspection<T>::size > 0)
     {
@@ -797,6 +527,15 @@ public:
       }
     }
 
+    // Copy output events
+    this->process_all_ports<process_after_run<safe_node_base, T>>();
+
+    // Clean up MIDI inputs
+    this->midi_buffers.clear_inputs(this->impl);
+
+    // Clean up sample-accurate control input ports
+    this->control_buffers.clear_inputs(this->impl);
+
     if constexpr(avnd::control_output_introspection<T>::size > 0)
     {
       if(this->control.outputs_set.any())
@@ -867,6 +606,7 @@ public:
     this->soundfiles.load(
         this->impl, hdl, avnd::predicate_index<N>{}, avnd::field_index<NField>{});
   }
+
   template <std::size_t N, std::size_t NField>
   void midifile_loaded(
       const std::shared_ptr<oscr::midifile_data>& hdl, avnd::predicate_index<N>,
@@ -875,6 +615,7 @@ public:
     this->midifiles.load(
         this->impl, hdl, avnd::predicate_index<N>{}, avnd::field_index<NField>{});
   }
+
 #if OSCR_HAS_MMAP_FILE_STORAGE
   template <std::size_t N, std::size_t NField>
   void file_loaded(
@@ -890,43 +631,47 @@ public:
 // FIXME these concepts are super messy
 
 template <typename FP, typename T>
-concept real_mono_processor = avnd::mono_per_sample_arg_processor<FP, T>
-                              || avnd::mono_per_sample_port_processor<FP, T>
-                              || avnd::monophonic_single_port_audio_effect<FP, T>
-                              || avnd::mono_per_channel_arg_processor<FP, T>
-                              || avnd::mono_per_channel_port_processor<FP, T>;
+concept real_mono_processor = !avnd::tag_cv<T>
+                              && (avnd::mono_per_sample_arg_processor<FP, T>
+                                  || avnd::mono_per_sample_port_processor<FP, T>
+                                  || avnd::monophonic_single_port_audio_effect<FP, T>
+                                  || avnd::mono_per_channel_arg_processor<FP, T>
+                                  || avnd::mono_per_channel_port_processor<FP, T>);
 template <typename T>
 concept real_good_mono_processor
     = real_mono_processor<float, T> || real_mono_processor<double, T>;
 
 template <typename T>
-concept mono_generator = avnd::monophonic_single_port_generator<float, T>
-                         || avnd::monophonic_single_port_generator<double, T>;
+concept mono_generator = !avnd::tag_cv<T>
+                         && (avnd::monophonic_single_port_generator<float, T>
+                             || avnd::monophonic_single_port_generator<double, T>);
 
 template <typename T>
 concept ossia_compatible_nonaudio_processor
-    = !(avnd::audio_argument_processor<T> || avnd::audio_port_processor<T>);
+    = avnd::tag_cv<T>
+      || !(avnd::audio_argument_processor<T> || avnd::audio_port_processor<T>);
 
 template <typename T>
 concept ossia_compatible_audio_processor
-    = avnd::poly_sample_array_input_port_count<double, T> > 0
-      || avnd::poly_sample_array_output_port_count<double, T> > 0;
+    = !avnd::tag_cv<T>
+      && (avnd::poly_sample_array_input_port_count<double, T> > 0
+          || avnd::poly_sample_array_output_port_count<double, T> > 0);
 
 template <typename T>
 class safe_node;
 
 struct tick_info
 {
+  const ossia::graph_node& self;
   const ossia::token_request& tk;
   ossia::exec_state_facade st;
   int64_t m_frames{};
 
   int64_t frames() const noexcept { return m_frames; }
+  int64_t start_in_flicks() const noexcept { return tk.prev_date.impl; }
+  int64_t end_in_flicks() const noexcept { return tk.date.impl; }
 
-  int64_t position_in_frames() const noexcept
-  {
-    return tk.start_date_to_physical(st.modelToSamples());
-  }
+  int64_t position_in_frames() const noexcept { return self.processed_frames(); }
 
   auto signature() const noexcept
   {
@@ -936,24 +681,38 @@ struct tick_info
     };
     return sig{.num = tk.signature.upper, .denom = tk.signature.lower};
   }
-  double speed() const noexcept { return tk.speed; };
-  double tempo() const noexcept { return tk.tempo; };
-  double relative_position() const noexcept { return tk.position(); };
-  double parent_duration() const noexcept { return tk.parent_duration.impl; };
-  double position_in_seconds() const noexcept
-  {
-    return position_in_nanoseconds() / 1e9;
-  };
+  double speed() const noexcept { return tk.speed; }
+  double tempo() const noexcept { return tk.tempo; }
+  double relative_position() const noexcept { return tk.position(); }
+  double parent_duration() const noexcept { return tk.parent_duration.impl; }
+  double position_in_seconds() const noexcept { return position_in_nanoseconds() / 1e9; }
   double position_in_nanoseconds() const noexcept
   {
     return (st.currentDate() - st.startDate());
-  };
+  }
+  double last_signature_change() const noexcept
+  {
+    return tk.musical_start_last_signature;
+  }
   double start_position_in_quarters() const noexcept
   {
     return tk.musical_start_position;
-  };
-  double end_position_in_quarters() const noexcept { return tk.musical_end_position; };
-  double bar_at_start() const noexcept { return tk.musical_start_last_bar; };
-  double bar_at_end() const noexcept { return tk.musical_end_last_bar; };
+  }
+  double end_position_in_quarters() const noexcept { return tk.musical_end_position; }
+  double bar_at_start() const noexcept { return tk.musical_start_last_bar; }
+  double bar_at_end() const noexcept { return tk.musical_end_last_bar; }
 };
 }
+
+namespace avnd
+{
+
+template <typename T, typename Tick>
+  requires(std::same_as<std::remove_cvref_t<typename T::tick>, ossia::token_request>)
+inline constexpr const ossia::token_request&
+get_tick_or_frames(avnd::effect_container<T>& implementation, const oscr::tick_info& v)
+{
+  return v.tk;
+}
+
+}
