@@ -1,6 +1,7 @@
 #pragma once
 #include <avnd/binding/ossia/from_value.hpp>
 #include <avnd/binding/ossia/geometry.hpp>
+#include <avnd/binding/ossia/port_base.hpp>
 #include <avnd/binding/ossia/window_functions.hpp>
 #include <avnd/common/struct_reflection.hpp>
 #include <avnd/concepts/fft.hpp>
@@ -40,45 +41,97 @@ struct process_before_run
   int frames{};
 
   template <avnd::parameter Field, std::size_t Idx>
-    requires(!avnd::control<Field>)
-  void init_value(
-      Field& ctrl, ossia::value_inlet& port, avnd::field_index<Idx> idx) const noexcept
+    requires ossia_port<Field>
+  void init_value(Field& ctrl, auto& port, avnd::field_index<Idx> idx) const noexcept
+  {
+  }
+  template <avnd::parameter Field, std::size_t Idx>
+    requires ossia_port<Field>
+  void operator()(Field& ctrl, auto& port, avnd::field_index<Idx>) const noexcept
   {
-    if(!port.data.get_data().empty())
-    {
-      auto& last = port.data.get_data().back().value;
-      update_value(self, impl, ctrl, last, ctrl.value, idx);
-    }
   }
 
   template <avnd::parameter Field, std::size_t Idx>
-    requires(avnd::control<Field>)
+    requires(!ossia_port<Field>)
   void init_value(
       Field& ctrl, ossia::value_inlet& port, avnd::field_index<Idx> idx) const noexcept
   {
+    // FIXME ensure this is not called for a dynamic_ports
     if(!port.data.get_data().empty())
     {
       auto& last = port.data.get_data().back().value;
       update_value(self, impl, ctrl, last, ctrl.value, idx);
 
-      // Get the index of the control in [0; N[
-      using type = typename Exec_T::processor_type;
-      using controls = avnd::control_input_introspection<type>;
-      constexpr int control_index = controls::field_index_to_index(idx);
+      if constexpr(avnd::control<Field>)
+      {
+        // Get the index of the control in [0; N[
+        using type = typename Exec_T::processor_type;
+        using controls = avnd::control_input_introspection<type>;
+        constexpr int control_index = controls::field_index_to_index(idx);
 
-      // Mark the control as changed
-      self.control.inputs_set.set(control_index);
+        // Mark the control as changed
+        self.control.inputs_set.set(control_index);
+      }
+    }
+  }
+
+  template <avnd::dynamic_ports_port Field, std::size_t Idx>
+  void init_value(
+      Field& ctrl, std::vector<ossia::value_inlet*>& ports,
+      avnd::field_index<Idx> idx) const noexcept
+  {
+    bool written = ports.size() != ctrl.ports.size();
+    ctrl.ports.resize(ports.size());
+    int p = 0;
+    for(auto pport : ports)
+    {
+      auto& port = *pport;
+      if(!port.data.get_data().empty())
+      {
+        auto& last = port.data.get_data().back().value;
+
+        // FIXME check optional ports case
+        written |= self.from_ossia_value(ctrl.ports[p], last, ctrl.ports[p].value, idx);
+
+        // FIXME
+        // if constexpr(avnd::control<Field>)
+        // {
+        //   // Get the index of the control in [0; N[
+        //   using type = typename Exec_T::processor_type;
+        //   using controls = avnd::control_input_introspection<type>;
+        //   constexpr int control_index = controls::field_index_to_index(idx);
+
+        //   // Mark the control as changed
+        //   self.control.inputs_set.set(control_index);
+        // }
+      }
+      ++p;
+    }
+
+    if(written)
+    {
+      if_possible(ctrl.update(impl));
     }
   }
 
   template <avnd::parameter Field, std::size_t Idx>
-    requires(!avnd::sample_accurate_parameter<Field> && !avnd::span_parameter<Field>)
+    requires(
+        !avnd::sample_accurate_parameter<Field> && !avnd::span_parameter<Field>
+        && !ossia_port<Field>)
   void operator()(
       Field& ctrl, ossia::value_inlet& port, avnd::field_index<Idx> idx) const noexcept
   {
     init_value(ctrl, port, avnd::field_index<Idx>{});
   }
 
+  template <avnd::dynamic_ports_port Field, std::size_t Idx>
+  void operator()(
+      Field& ctrl, std::vector<ossia::value_inlet*>& port,
+      avnd::field_index<Idx> idx) const noexcept
+  {
+    init_value(ctrl, port, avnd::field_index<Idx>{});
+  }
+
   template <avnd::linear_sample_accurate_parameter Field, std::size_t Idx>
   void operator()(
       Field& ctrl, ossia::value_inlet& port, avnd::field_index<Idx>) const noexcept
@@ -130,7 +183,7 @@ struct process_before_run
     {
       using sc_in = avnd::spectrum_split_channel_input_introspection<
           typename Exec_T::processor_type>;
-      constexpr auto fft_idx = sc_in::field_index_to_index(idx);
+      static constexpr auto fft_idx = sc_in::field_index_to_index(idx);
       ossia::fft& fft = get<fft_idx>(self.spectrums.split_channel.ffts);
 
       auto& samples = port.data.channel(0);
@@ -156,7 +209,7 @@ struct process_before_run
     {
       using sc_in = avnd::spectrum_complex_channel_input_introspection<
           typename Exec_T::processor_type>;
-      constexpr auto fft_idx = sc_in::field_index_to_index(idx);
+      static constexpr auto fft_idx = sc_in::field_index_to_index(idx);
       ossia::fft& fft = get<fft_idx>(self.spectrums.complex_channel.ffts);
 
       auto& samples = port.data.channel(0);
@@ -178,7 +231,7 @@ struct process_before_run
     {
       using sc_in = avnd::spectrum_split_bus_input_introspection<
           typename Exec_T::processor_type>;
-      constexpr auto fft_idx = sc_in::field_index_to_index(idx);
+      static constexpr auto fft_idx = sc_in::field_index_to_index(idx);
       auto& ffts = get<fft_idx>(self.spectrums.split_bus.ffts);
 
       for(std::size_t c = 0; c < port.data.channels(); c++)
@@ -208,7 +261,7 @@ struct process_before_run
     {
       using sc_in = avnd::spectrum_split_bus_input_introspection<
           typename Exec_T::processor_type>;
-      constexpr auto fft_idx = sc_in::field_index_to_index(idx);
+      static constexpr auto fft_idx = sc_in::field_index_to_index(idx);
       auto& ffts = get<fft_idx>(self.spectrums.complex_bus.ffts);
 
       for(std::size_t c = 0; c < port.data.channels(); c++)
@@ -261,8 +314,8 @@ struct process_before_run
 
     using span_in = avnd::span_parameter_input_introspection<Obj_T>;
     using T = typename decltype(Field::value)::value_type;
-    constexpr auto field_idx
-        = span_in::template field_index_to_index(avnd::field_index<Idx>{});
+    static constexpr auto field_idx
+        = span_in::field_index_to_index(avnd::field_index<Idx>{});
     auto& dest_vec = get<field_idx>(self.control_buffers.span_inputs);
     dest_vec.resize(src_vec->size());
 
@@ -340,7 +393,7 @@ struct process_before_run
   operator()(Field& ctrl, ossia::midi_inlet& port, avnd::field_index<Idx>) const noexcept
   {
     using midi_msg_type =
-        typename std::decay_t<decltype(Field::midi_messages)>::value_type;
+        typename std::remove_cvref_t<decltype(Field::midi_messages)>::value_type;
 
     if constexpr(std::is_same_v<midi_msg_type, libremidi::message>)
     {
@@ -369,6 +422,19 @@ struct process_before_run
     }
   }
 
+  template <avnd::dynamic_ports_port Field, std::size_t Idx>
+  void operator()(
+      Field& ctrl, std::vector<ossia::value_outlet*>& ports,
+      avnd::field_index<Idx>) const noexcept
+  {
+    ctrl.ports.resize(ports.size());
+    for(auto& port_value : ctrl.ports)
+    {
+      using type = std::remove_cvref_t<decltype(port_value.value)>;
+      port_value.value = type{};
+    }
+  }
+
   template <avnd::control Field, std::size_t Idx>
     requires(!avnd::sample_accurate_control<Field>)
   void operator()(
@@ -377,6 +443,7 @@ struct process_before_run
     if constexpr(avnd::optional_ish<decltype(Field::value)>)
       ctrl.value = {};
   }
+
   template <avnd::value_port Field, std::size_t Idx>
     requires(!avnd::sample_accurate_value_port<Field>)
   void operator()(
@@ -385,6 +452,7 @@ struct process_before_run
     if constexpr(avnd::optional_ish<decltype(Field::value)>)
       ctrl.value = {};
   }
+
   template <avnd::sample_accurate_control Field, std::size_t Idx>
   void operator()(
       Field& ctrl, ossia::value_outlet& port, avnd::field_index<Idx>) const noexcept
