@@ -1,3 +1,10 @@
+//
+// Copyright (c) 2023-2024 Ivica Siladic, Bruno Iljazovic, Korina Simicevic
+//
+// Distributed under the Boost Software License, Version 1.0.
+// (See accompanying file LICENSE or copy at http://www.boost.org/LICENSE_1_0.txt)
+//
+
 //[hello_world_over_tcp
 #include <iostream>
 
@@ -14,12 +21,12 @@ int main() {
 	async_mqtt5::mqtt_client<boost::asio::ip::tcp::socket> client(ioc);
 
 	// 1883 is the default TCP MQTT port.
-	client.brokers("<your-mqtt-broker>", 1883)
+	client.brokers("broker.hivemq.com", 1883)
 		.async_run(boost::asio::detached);
 
 	client.async_publish<async_mqtt5::qos_e::at_most_once>(
-		"<topic>", "Hello world!",
-		async_mqtt5::retain_e::no, async_mqtt5::publish_props {},
+		"async-mqtt5/test", "Hello world!",
+		async_mqtt5::retain_e::yes, async_mqtt5::publish_props {},
 		[&client](async_mqtt5::error_code ec) {
 			std::cout << ec.message() << std::endl;
 			client.async_disconnect(boost::asio::detached);
