@@ -1,3 +1,10 @@
+//
+// Copyright (c) 2023-2024 Ivica Siladic, Bruno Iljazovic, Korina Simicevic
+//
+// Distributed under the Boost Software License, Version 1.0.
+// (See accompanying file LICENSE or copy at http://www.boost.org/LICENSE_1_0.txt)
+//
+
 //[hello_world_over_websocket_tcp
 #include <iostream>
 
@@ -12,13 +19,13 @@
 int main() {
 	boost::asio::io_context ioc;
 
-	// Construct the Client with ``[beastreflink boost__beast__websocket__stream websocket::stream<__TCP_SOCKET__>]`` as the underlying stream.
+	// Construct the Client with WebSocket/TCP as the underlying stream.
 	async_mqtt5::mqtt_client<
 		boost::beast::websocket::stream<boost::asio::ip::tcp::socket>
 	> client(ioc);
 
 	// 8083 is the default Webscoket/TCP MQTT port.
-	client.brokers("<your-mqtt-broker>", 8083)
+	client.brokers("<your-mqtt-broker-path>", 8083) // Path example: localhost/mqtt
 		.async_run(boost::asio::detached);
 
 	client.async_publish<async_mqtt5::qos_e::at_most_once>(
