@@ -1,14 +1,21 @@
+//
+// Copyright (c) 2023-2024 Ivica Siladic, Bruno Iljazovic, Korina Simicevic
+//
+// Distributed under the Boost Software License, Version 1.0.
+// (See accompanying file LICENSE or copy at http://www.boost.org/LICENSE_1_0.txt)
+//
+
 #ifndef ASYNC_MQTT5_TRAITS_HPP
 #define ASYNC_MQTT5_TRAITS_HPP
 
 #include <optional>
 #include <vector>
+#include <utility>
 
 #include <boost/container/small_vector.hpp>
 #include <boost/range/iterator_range_core.hpp>
 #include <boost/type_traits/remove_cv_ref.hpp>
 
-
 namespace async_mqtt5 {
 
 template <typename>
@@ -51,8 +58,6 @@ constexpr bool is_boost_iterator = is_specialization<
 	boost::remove_cv_ref_t<T>, boost::iterator_range
 >;
 
-
-
 } // end namespace async_mqtt5
 
 #endif // !ASYNC_MQTT5_TRAITS_HPP
