@@ -1,17 +1,32 @@
+//
+// Copyright (c) 2023-2024 Ivica Siladic, Bruno Iljazovic, Korina Simicevic
+//
+// Distributed under the Boost Software License, Version 1.0.
+// (See accompanying file LICENSE or copy at http://www.boost.org/LICENSE_1_0.txt)
+//
+
 #ifndef ASYNC_MQTT5_MQTT_CLIENT_HPP
 #define ASYNC_MQTT5_MQTT_CLIENT_HPP
 
+#include <memory>
+#include <string>
+#include <type_traits>
+#include <variant> // std::monostate
+#include <vector>
+
+#include <boost/asio/async_result.hpp>
 #include <boost/system/error_code.hpp>
 
 #include <async_mqtt5/error.hpp>
 #include <async_mqtt5/types.hpp>
 
+#include <async_mqtt5/detail/rebind_executor.hpp>
+
 #include <async_mqtt5/impl/client_service.hpp>
 #include <async_mqtt5/impl/publish_send_op.hpp>
-#include <async_mqtt5/impl/read_message_op.hpp>
+#include <async_mqtt5/impl/re_auth_op.hpp>
 #include <async_mqtt5/impl/subscribe_op.hpp>
 #include <async_mqtt5/impl/unsubscribe_op.hpp>
-#include <async_mqtt5/impl/re_auth_op.hpp>
 
 namespace async_mqtt5 {
 
@@ -39,6 +54,17 @@ class mqtt_client {
 public:
 	/// The executor type associated with the client.
 	using executor_type = typename StreamType::executor_type;
+
+	/// Rebinds the client type to another executor.
+	template <typename Executor>
+	struct rebind_executor {
+		/// The client type when rebound to the specified executor.
+		using other = mqtt_client<
+			typename detail::rebind_executor<StreamType, Executor>::other,
+			TlsContext
+		>;
+	};
+
 private:
 	using stream_type = StreamType;
 	using tls_context_type = TlsContext;
@@ -96,7 +122,7 @@ public:
 	 *
 	 * \details Moved-from client can only be destructed
 	 */
-	mqtt_client(mqtt_client&& other) noexcept = default;
+	mqtt_client(mqtt_client&&) noexcept = default;
 
 
 	/**
@@ -163,9 +189,9 @@ public:
 	 *
 	 * \par Completion condition
 	 * The asynchronous operation will complete with
-	 * `boost::asio::error::operation_aborted` when client is cancelled by calling
+	 * `boost::asio::error::operation_aborted` when the client is cancelled by calling
 	 * \ref mqtt_client::async_disconnect, \ref mqtt_client::cancel, destruction or
-	 * if non-recoverable error happens during connection attempt (e.g. access denied).
+	 * if a non-recoverable error happens during a connection attempt (e.g. access denied).
 	 *
 	 *	\par Error codes
 	 *	The list of all possible error codes that this operation can finish with:\n
@@ -175,20 +201,14 @@ public:
 	 *	This asynchronous operation supports cancellation for the following \__CANCELLATION_TYPE\__ values:\n
 	 *		- `cancellation_type::terminal` - invokes \ref mqtt_client::cancel \n
 	 */
-	template <typename CompletionToken>
-	decltype(auto) async_run(CompletionToken&& token) {
-		using Signature = void(error_code);
-
-		auto initiation = [] (auto handler, const impl_type& impl) {
-			impl->run(std::move(handler));
-
-			detail::ping_op { impl }.perform();
-			detail::read_message_op { impl }.perform();
-			detail::sentry_op { impl }.perform();
-		};
-
+	template <
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
+	decltype(auto) async_run(CompletionToken&& token = {}) {
+		using Signature = void (error_code);
 		return asio::async_initiate<CompletionToken, Signature>(
-			initiation, token, _impl
+			detail::initiate_async_run(_impl), token
 		);
 	}
 
@@ -476,31 +496,20 @@ public:
 	 *		- `cancellation_type::partial` & `cancellation_type::total` - prevents potential resending of the \__PUBLISH\__ packet \n
 	 *
 	 */
-	template <qos_e qos_type, typename CompletionToken>
+	template <qos_e qos_type, 
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
 	decltype(auto) async_publish(
 		std::string topic, std::string payload,
 		retain_e retain, const publish_props& props,
-		CompletionToken&& token
+		CompletionToken&& token = {}
 	) {
 		using Signature = detail::on_publish_signature<qos_type>;
-
-		auto initiation = [] (
-			auto handler, std::string topic, std::string payload,
-			retain_e retain, const publish_props& props,
-			const impl_type& impl
-		) {
-			detail::publish_send_op<
-				client_service_type, decltype(handler), qos_type
-			> { impl, std::move(handler) }
-				.perform(
-					std::move(topic), std::move(payload),
-					retain, props
-				);
-		};
-
 		return asio::async_initiate<CompletionToken, Signature>(
-			initiation, token,
-			std::move(topic), std::move(payload), retain, props, _impl
+			detail::initiate_async_publish<client_service_type, qos_type>(_impl),
+			token,
+			std::move(topic), std::move(payload), retain, props
 		);
 	}
 
@@ -559,32 +568,27 @@ public:
 	 *		- `cancellation_type::partial` & `cancellation_type::total` - prevents potential resending of the \__SUBSCRIBE\__ packet \n
 	 *
 	 */
-	template <typename CompletionToken>
+	template <
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
 	decltype(auto) async_subscribe(
 		const std::vector<subscribe_topic>& topics,
 		const subscribe_props& props,
-		CompletionToken&& token
+		CompletionToken&& token = {}
 	) {
 		using Signature = void (
 			error_code, std::vector<reason_code>, suback_props
 		);
-
-		auto initiation = [] (
-			auto handler, const std::vector<subscribe_topic>& topics,
-			const subscribe_props& props, const impl_type& impl
-		) {
-			detail::subscribe_op { impl, std::move(handler) }
-				.perform(topics, props);
-		};
-
 		return asio::async_initiate<CompletionToken, Signature>(
-			initiation, token, topics, props, _impl
+			detail::initiate_async_subscribe(_impl), token,
+			topics, props
 		);
 	}
 
 	/**
 	 * \brief Send a \__SUBSCRIBE\__ packet to Broker to create a subscription
-	 * to one Topics of interest.
+	 * to one Topic of interest.
 	 *
 	 * \details After the subscription has been established, the Broker will send
 	 * \__PUBLISH\__ packets to the Client to forward Application Messages that were published
@@ -637,10 +641,13 @@ public:
 	 *		- `cancellation_type::partial` & `cancellation_type::total` - prevents potential resending of the \__SUBSCRIBE\__ packet \n
 	 *
 	 */
-	template <typename CompletionToken>
+	template <
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
 	decltype(auto) async_subscribe(
 		const subscribe_topic& topic, const subscribe_props& props,
-		CompletionToken&& token
+		CompletionToken&& token = {}
 	) {
 		return async_subscribe(
 			std::vector<subscribe_topic> { topic }, props,
@@ -700,26 +707,20 @@ public:
 	 *		- `cancellation_type::partial` & `cancellation_type::total` - prevents potential resending of the \__UNSUBSCRIBE\__ packet \n
 	 *
 	 */
-	template <typename CompletionToken>
+	template <
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
 	decltype(auto) async_unsubscribe(
 		const std::vector<std::string>& topics,	const unsubscribe_props& props,
-		CompletionToken&& token
+		CompletionToken&& token = {}
 	) {
 		using Signature = void (
 			error_code, std::vector<reason_code>, unsuback_props
 		);
-
-		auto initiation = [](
-			auto handler,
-			const std::vector<std::string>& topics,
-			const unsubscribe_props& props,	const impl_type& impl
-		) {
-			detail::unsubscribe_op { impl, std::move(handler) }
-				.perform(topics, props);
-		};
-
 		return asio::async_initiate<CompletionToken, Signature>(
-			initiation, token, topics, props, _impl
+			detail::initiate_async_unsubscribe(_impl), token,
+			topics, props
 		);
 	}
 
@@ -774,10 +775,13 @@ public:
 	 *		- `cancellation_type::partial` & `cancellation_type::total` - prevents potential resending of the \__UNSUBSCRIBE\__ packet \n
 	 *
 	 */
-	template <typename CompletionToken>
+	template <
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
 	decltype(auto) async_unsubscribe(
 		const std::string& topic, const unsubscribe_props& props,
-		CompletionToken&& token
+		CompletionToken&& token = {}
 	) {
 		return async_unsubscribe(
 			std::vector<std::string> { topic }, props,
@@ -791,7 +795,7 @@ public:
 	 *
 	 * \details The Client will receive and complete deliveries for all the
 	 * \__PUBLISH\__ packets received from the Broker throughout its lifetime.
-	 * The Client will store them internally in order they were delivered.
+	 * The Client will store them internally in the order they were delivered.
 	 * Calling this function will attempt to receive an Application Message
 	 * from internal storage.
 	 *
@@ -834,12 +838,12 @@ public:
 	 *		- `cancellation_type::partial` \n
 	 *		- `cancellation_type::total` \n
 	 */
-	template <typename CompletionToken>
-	decltype(auto) async_receive(CompletionToken&& token) {
-		// Sig = void (error_code, std::string, std::string, publish_props)
-		return _impl->async_channel_receive(
-			std::forward<CompletionToken>(token)
-		);
+	template <
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
+	decltype(auto) async_receive(CompletionToken&& token = {}) {
+		return _impl->async_channel_receive(std::forward<CompletionToken>(token));
 	}
 
 	/**
@@ -855,7 +859,7 @@ public:
 	 * See \ref mqtt_client::cancel.
 	 *
 	 * \param reason_code Reason Code to notify
-	 * the Broker of the reason for disconnection.
+	 * the Broker of the reason for the disconnection.
 	 * \param props An instance of \__DISCONNECT_PROPS\__.
 	 * \param token Completion token that will be used to produce a
 	 * completion handler. The handler will be invoked when the operation completes.
@@ -880,7 +884,7 @@ public:
 	 *	The list of all possible error codes that this operation can finish with:\n
 	 *		- `boost::system::errc::errc_t::success`\n
 	 *		- `boost::asio::error::operation_aborted`[footnote
-				This error code can appear if the Client failed to send the \__DISCONNECT\__ packet to the Server.
+				This error code can appear if the Client fails to send the \__DISCONNECT\__ packet to the Server.
 				Regardless, the connection to the Server is terminated, and the Client is cancelled.
 			]\n
 	 *		- \link async_mqtt5::client::error::malformed_packet \endlink
@@ -892,10 +896,13 @@ public:
 	 *		- `cancellation_type::terminal` - invokes \ref mqtt_client::cancel \n
 	 * 
 	 */
-	template <typename CompletionToken>
+	template <
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
 	decltype(auto) async_disconnect(
 		disconnect_rc_e reason_code, const disconnect_props& props,
-		CompletionToken&& token
+		CompletionToken&& token = {}
 	) {
 		auto impl = _impl;
 		_impl = impl->dup();
@@ -941,7 +948,7 @@ public:
 	 *	The list of all possible error codes that this operation can finish with:\n
 	 *		- `boost::system::errc::errc_t::success`\n
 	 *		- `boost::asio::error::operation_aborted`[footnote
-				This error code can appear if the Client failed to send the \__DISCONNECT\__ packet to the Server.
+				This error code can appear if the Client fails to send the \__DISCONNECT\__ packet to the Server.
 				Regardless, the connection to the Server is terminated, and the Client is cancelled.
 			]\n
 	 *		- \link async_mqtt5::client::error::malformed_packet \endlink
@@ -952,13 +959,17 @@ public:
 	 *	This asynchronous operation supports cancellation for the following \__CANCELLATION_TYPE\__ values:\n
 	 *		- `cancellation_type::terminal` - invokes \ref mqtt_client::cancel \n
 	 */
-	template <typename CompletionToken>
-	decltype(auto) async_disconnect(CompletionToken&& token) {
+	template <
+		typename CompletionToken =
+			typename asio::default_completion_token<executor_type>::type
+	>
+	decltype(auto) async_disconnect(CompletionToken&& token = {}) {
 		return async_disconnect(
 			disconnect_rc_e::normal_disconnection,
 			disconnect_props {}, std::forward<CompletionToken>(token)
 		);
 	}
+
 };
 
 
