@@ -1,6 +1,14 @@
+//
+// Copyright (c) 2023-2024 Ivica Siladic, Bruno Iljazovic, Korina Simicevic
+//
+// Distributed under the Boost Software License, Version 1.0.
+// (See accompanying file LICENSE or copy at http://www.boost.org/LICENSE_1_0.txt)
+//
+
 #include <boost/test/unit_test.hpp>
 
-#include <boost/asio/io_context.hpp>
+#include <chrono>
+
 #include <boost/asio/steady_timer.hpp>
 
 #include <async_mqtt5/impl/reconnect_op.hpp>
