@@ -11,11 +11,9 @@
 #include <chrono>
 #include <cstdlib>
 #include <iostream>
-#include <map>
 #include <thread>
 
 int main()
-try
 {
   for (auto& api : libremidi::available_apis())
   {
@@ -25,29 +23,28 @@ try
     // On Windows 10, apparently the MIDI devices aren't exactly available as soon as the app open...
     std::this_thread::sleep_for(std::chrono::milliseconds(100));
 
-    libremidi::observer midi{{}, libremidi::observer_configuration_for(api)};
+    libremidi::observer midi{
+        {.track_hardware = true, .track_virtual = true},
+        libremidi::observer_configuration_for(api)};
     {
       // Check inputs.
       auto ports = midi.get_input_ports();
       std::cout << ports.size() << " MIDI input sources:\n";
+      int i = 0;
       for (auto& port : ports)
-        std::cout << " - " << port << '\n';
+        std::cout << " - " << i++ << ": " << port << '\n';
     }
 
     {
       // Check outputs.
       auto ports = midi.get_output_ports();
       std::cout << ports.size() << " MIDI output sinks:\n";
+      int i = 0;
       for (auto& port : ports)
-        std::cout << " - " << port << '\n';
+        std::cout << " - " << i++ << ": " << port << '\n';
     }
 
     std::cout << "\n";
   }
   return 0;
 }
-catch (const std::exception& error)
-{
-  std::cerr << error.what() << std::endl;
-  return EXIT_FAILURE;
-}
