@@ -60,11 +60,12 @@ convert_midi2_to_midi1_input_configuration(const ump_input_configuration& base_c
   return c2;
 }
 
-LIBREMIDI_INLINE auto make_midi_in(auto base_conf, std::any api_conf, auto backends)
+static LIBREMIDI_INLINE std::unique_ptr<midi_in_api>
+make_midi_in(auto base_conf, std::any api_conf, auto backends)
 {
   std::unique_ptr<midi_in_api> ptr;
 
-  assert(base_conf.on_message);
+  assert(base_conf.on_message || base_conf.on_raw_data);
 
   auto from_api = [&]<typename T>(T& /*backend*/) mutable {
     if (auto conf = std::any_cast<typename T::midi_in_configuration>(&api_conf))
@@ -78,20 +79,22 @@ LIBREMIDI_INLINE auto make_midi_in(auto base_conf, std::any api_conf, auto backe
   return ptr;
 }
 
-LIBREMIDI_INLINE midi_in::midi_in(const input_configuration& base_conf) noexcept
+/// MIDI 1 helpers
+static LIBREMIDI_INLINE std::unique_ptr<midi_in_api>
+make_midi1_in(const input_configuration& base_conf)
 {
   for (const auto& api : available_apis())
   {
     try
     {
-      impl_ = make_midi_in(base_conf, midi_in_configuration_for(api), midi1::available_backends);
+      auto impl
+          = make_midi_in(base_conf, midi_in_configuration_for(api), midi1::available_backends);
+      if (impl)
+        return impl;
     }
     catch (const std::exception& e)
     {
     }
-
-    if (impl_)
-      return;
   }
 
   // No MIDI 1 backend, try the MIDI 2 ones with a wrap:
@@ -101,24 +104,70 @@ LIBREMIDI_INLINE midi_in::midi_in(const input_configuration& base_conf) noexcept
     {
       try
       {
-        impl_ = make_midi_in(c2, midi_in_configuration_for(api), midi2::available_backends);
+        auto impl = make_midi_in(c2, midi_in_configuration_for(api), midi2::available_backends);
+        if (impl)
+          return impl;
       }
       catch (const std::exception& e)
       {
       }
-
-      if (impl_)
-        return;
     }
   }
 
-  if (!impl_)
-    impl_ = std::make_unique<midi_in_dummy>(input_configuration{}, dummy_configuration{});
+  return std::make_unique<midi_in_dummy>(input_configuration{}, dummy_configuration{});
+}
+
+static LIBREMIDI_INLINE std::unique_ptr<midi_in_api>
+make_midi1_in(const input_configuration& base_conf, const std::any& api_conf, libremidi::API api)
+{
+  if (libremidi::is_midi1(api))
+  {
+    return make_midi_in(base_conf, api_conf, midi1::available_backends);
+  }
+  else if (libremidi::is_midi2(api))
+  {
+    auto c2 = convert_midi1_to_midi2_input_configuration(base_conf);
+    return make_midi_in(c2, api_conf, midi2::available_backends);
+  }
+  return {};
+}
+
+static LIBREMIDI_INLINE std::unique_ptr<midi_in_api>
+make_midi1_in(const input_configuration& base_conf, const std::any& api_conf)
+{
+  if (!api_conf.has_value())
+  {
+    return make_midi1_in(base_conf);
+  }
+  else if (auto api_p = std::any_cast<libremidi::API>(&api_conf))
+  {
+    if (*api_p == libremidi::API::UNSPECIFIED)
+    {
+      return make_midi1_in(base_conf);
+    }
+    else
+    {
+      return make_midi1_in(base_conf, midi_in_configuration_for(*api_p), *api_p);
+    }
+  }
+  else
+  {
+    if (auto api = libremidi::midi_api(api_conf); api == libremidi::API::UNSPECIFIED)
+      return {};
+    else
+      return make_midi1_in(base_conf, api_conf, libremidi::midi_api(api_conf));
+  }
+}
+
+/// MIDI 1 constructors
+LIBREMIDI_INLINE midi_in::midi_in(const input_configuration& base_conf) noexcept
+    : impl_{make_midi1_in(base_conf)}
+{
 }
 
 LIBREMIDI_INLINE
 midi_in::midi_in(input_configuration base_conf, std::any api_conf)
-    : impl_{make_midi_in(base_conf, api_conf, midi1::available_backends)}
+    : impl_{make_midi1_in(base_conf, api_conf)}
 {
   if (!impl_)
   {
@@ -128,20 +177,21 @@ midi_in::midi_in(input_configuration base_conf, std::any api_conf)
   }
 }
 
-LIBREMIDI_INLINE midi_in::midi_in(ump_input_configuration base_conf) noexcept
+/// MIDI 2 helpers
+static LIBREMIDI_INLINE std::unique_ptr<midi_in_api>
+make_midi2_in(const ump_input_configuration& base_conf)
 {
   for (const auto& api : available_ump_apis())
   {
     try
     {
-      impl_ = make_midi_in(base_conf, midi_in_configuration_for(api), midi2::available_backends);
+      if (auto ret
+          = make_midi_in(base_conf, midi_in_configuration_for(api), midi2::available_backends))
+        return ret;
     }
     catch (const std::exception& e)
     {
     }
-
-    if (impl_)
-      return;
   }
 
   // No MIDI 2 backend, try the MIDI 1 ones with a wrap:
@@ -151,24 +201,70 @@ LIBREMIDI_INLINE midi_in::midi_in(ump_input_configuration base_conf) noexcept
     {
       try
       {
-        impl_ = make_midi_in(c2, midi_in_configuration_for(api), midi1::available_backends);
+        if (auto ret = make_midi_in(c2, midi_in_configuration_for(api), midi1::available_backends))
+          return ret;
       }
       catch (const std::exception& e)
       {
       }
-
-      if (impl_)
-        return;
     }
   }
 
-  if (!impl_)
-    impl_ = std::make_unique<midi_in_dummy>(input_configuration{}, dummy_configuration{});
+  return {};
+}
+
+static LIBREMIDI_INLINE std::unique_ptr<midi_in_api> make_midi2_in(
+    const ump_input_configuration& base_conf, const std::any& api_conf, libremidi::API api)
+{
+  if (is_midi2(api))
+  {
+    return make_midi_in(base_conf, api_conf, midi2::available_backends);
+  }
+  else if (is_midi1(api))
+  {
+    auto c2 = convert_midi2_to_midi1_input_configuration(base_conf);
+    return make_midi_in(c2, api_conf, midi1::available_backends);
+  }
+
+  return {};
+}
+
+static LIBREMIDI_INLINE std::unique_ptr<midi_in_api>
+make_midi2_in(const ump_input_configuration& base_conf, const std::any& api_conf)
+{
+  if (!api_conf.has_value())
+  {
+    return make_midi2_in(base_conf);
+  }
+  else if (auto api_p = std::any_cast<libremidi::API>(&api_conf))
+  {
+    if (*api_p == libremidi::API::UNSPECIFIED)
+    {
+      return make_midi2_in(base_conf);
+    }
+    else
+    {
+      return make_midi2_in(base_conf, midi_in_configuration_for(*api_p), *api_p);
+    }
+  }
+  else
+  {
+    if (auto api = libremidi::midi_api(api_conf); api == libremidi::API::UNSPECIFIED)
+      return {};
+    else
+      return make_midi2_in(base_conf, api_conf, libremidi::midi_api(api_conf));
+  }
+}
+
+/// MIDI 2 constructors
+LIBREMIDI_INLINE midi_in::midi_in(ump_input_configuration base_conf) noexcept
+    : impl_{make_midi2_in(base_conf)}
+{
 }
 
 LIBREMIDI_INLINE
 midi_in::midi_in(ump_input_configuration base_conf, std::any api_conf)
-    : impl_{make_midi_in(base_conf, api_conf, midi2::available_backends)}
+    : impl_{make_midi2_in(base_conf, api_conf)}
 {
   if (!impl_)
   {
