@@ -10,7 +10,8 @@
 
 namespace libremidi
 {
-LIBREMIDI_INLINE auto make_midi_out(auto base_conf, std::any api_conf)
+static LIBREMIDI_INLINE std::unique_ptr<midi_out_api>
+make_midi_out_impl(auto base_conf, std::any api_conf)
 {
   std::unique_ptr<midi_out_api> ptr;
   auto from_api = [&]<typename T>(T& /*backend*/) mutable {
@@ -26,24 +27,66 @@ LIBREMIDI_INLINE auto make_midi_out(auto base_conf, std::any api_conf)
   return ptr;
 }
 
-LIBREMIDI_INLINE midi_out::midi_out(const output_configuration& base_conf) noexcept
+static LIBREMIDI_INLINE std::unique_ptr<midi_out_api>
+make_midi_out(const output_configuration& base_conf)
 {
   for (const auto& api : available_apis())
   {
     try
     {
-      impl_ = make_midi_out(base_conf, midi_out_configuration_for(api));
+      if (auto ret = make_midi_out_impl(base_conf, midi_out_configuration_for(api)))
+        return ret;
     }
     catch (const std::exception& e)
     {
     }
-
-    if (impl_)
-      return;
   }
 
-  if (!impl_)
-    impl_ = std::make_unique<midi_out_dummy>(output_configuration{}, dummy_configuration{});
+  for (const auto& api : available_ump_apis())
+  {
+    try
+    {
+      if (auto ret = make_midi_out_impl(base_conf, midi_out_configuration_for(api)))
+        return ret;
+    }
+    catch (const std::exception& e)
+    {
+    }
+  }
+
+  return std::make_unique<midi_out_dummy>(output_configuration{}, dummy_configuration{});
+}
+
+static LIBREMIDI_INLINE std::unique_ptr<midi_out_api>
+make_midi_out(const output_configuration& base_conf, const std::any& api_conf)
+{
+  if (!api_conf.has_value())
+  {
+    return make_midi_out(base_conf);
+  }
+  else if (auto api_p = std::any_cast<libremidi::API>(&api_conf))
+  {
+    if (*api_p == libremidi::API::UNSPECIFIED)
+    {
+      return make_midi_out(base_conf);
+    }
+    else
+    {
+      return make_midi_out_impl(base_conf, midi_out_configuration_for(*api_p));
+    }
+  }
+  else
+  {
+    if (auto api = libremidi::midi_api(api_conf); api == libremidi::API::UNSPECIFIED)
+      return make_midi_out(base_conf);
+    else
+      return make_midi_out_impl(base_conf, api_conf);
+  }
+}
+
+LIBREMIDI_INLINE midi_out::midi_out(const output_configuration& base_conf) noexcept
+    : impl_{make_midi_out(base_conf)}
+{
 }
 
 LIBREMIDI_INLINE
@@ -187,6 +230,12 @@ stdx::error midi_out::send_message(const unsigned char* message, size_t size) co
   return impl_->send_message(message, size);
 }
 
+LIBREMIDI_INLINE
+int64_t midi_out::current_time()
+{
+  return impl_->current_time();
+}
+
 LIBREMIDI_INLINE
 stdx::error midi_out::schedule_message(int64_t ts, const unsigned char* message, size_t size) const
 {
@@ -243,6 +292,28 @@ stdx::error midi_out::send_ump(uint32_t b0, uint32_t b1, uint32_t b2, uint32_t b
   return send_ump(std::to_array({b0, b1, b2, b3}));
 }
 
+LIBREMIDI_INLINE
+stdx::error midi_out::send_ump(int32_t b0) const
+{
+  return send_ump(reinterpret_cast<uint32_t*>(&b0), 1);
+}
+
+LIBREMIDI_INLINE
+stdx::error midi_out::send_ump(int64_t b01) const
+{
+  auto ptr = reinterpret_cast<uint32_t*>(&b01);
+  uint32_t msg[2]{ptr[1], ptr[0]};
+  return send_ump(msg, 2);
+}
+
+LIBREMIDI_INLINE
+stdx::error midi_out::send_ump(uint64_t b01) const
+{
+  auto ptr = reinterpret_cast<uint32_t*>(&b01);
+  uint32_t msg[2]{ptr[1], ptr[0]};
+  return send_ump(msg, 2);
+}
+
 LIBREMIDI_INLINE
 stdx::error midi_out::schedule_ump(int64_t ts, const uint32_t* message, size_t size) const
 {
