@@ -1,5 +1,6 @@
 #pragma once
 #include <ossia/detail/logger.hpp>
+#include <ossia/network/context.hpp>
 #include <ossia/network/sockets/configuration.hpp>
 
 #include <boost/asio/io_context.hpp>
@@ -131,6 +132,11 @@ public:
   {
   }
 
+  unix_stream_server(const fd_configuration& conf, ossia::net::network_context_ptr ctx)
+      : unix_stream_server{conf, ctx->context}
+  {
+  }
+
   boost::asio::io_context& m_context;
   proto::acceptor m_acceptor;
 };
