@@ -65,16 +65,15 @@ std::string
 midi_protocol::get_midi_port_name(ossia::net::device_base* dev, const midi_info& info)
 {
   std::string name{};
+  const bool input = info.type == midi_info::Type::Input;
   if(dev)
-    name = dev->get_name();
+    name = (input ? "i " : "o ") + dev->get_name();
   else
   {
     if(info.is_virtual)
-      name = info.type == midi_info::Type::Input ? "libossia MIDI virtual input"
-                                                 : "libossia MIDI virtual output";
+      name = input ? "i libossia virtual" : "o libossia virtual";
     else
-      name = info.type == midi_info::Type::Input ? "libossia MIDI input"
-                                                 : "libossia MIDI output";
+      name = input ? "i libossia" : "o libossia";
   }
   return name;
 }
@@ -746,7 +745,11 @@ std::vector<midi_info> midi_protocol::scan(libremidi::API api)
 
   std::vector<midi_info> vec;
 
-  libremidi::observer in{{}, libremidi::observer_configuration_for(api)};
+  libremidi::observer_configuration conf{};
+  conf.track_hardware = true;
+  conf.track_virtual = true;
+
+  libremidi::observer in{conf, libremidi::observer_configuration_for(api)};
 
   for(auto& port : in.get_input_ports())
     vec.emplace_back(port, false);
