@@ -7,7 +7,6 @@
 #include <ossia/network/common/node_visitor.hpp>
 #include <ossia/network/context.hpp>
 #include <ossia/network/exceptions.hpp>
-#include <ossia/network/http/http_client.hpp>
 #include <ossia/network/osc/detail/osc.hpp>
 #include <ossia/network/osc/detail/osc_1_1_extended_policy.hpp>
 #include <ossia/network/osc/detail/osc_protocol_common.hpp>
@@ -22,9 +21,8 @@
 #include <ossia/network/oscquery/detail/value_to_json.hpp>
 #include <ossia/network/sockets/udp_socket.hpp>
 #include <ossia/network/sockets/websocket_client.hpp>
+#include <ossia/protocols/oscquery/http_requests.hpp>
 
-#include <boost/algorithm/string.hpp>
-#include <boost/algorithm/string/erase.hpp>
 namespace ossia::oscquery_asio
 {
 struct oscquery_mirror_asio_protocol::osc_sender_impl : ossia::net::udp_send_socket
@@ -36,86 +34,6 @@ struct oscquery_mirror_asio_protocol::osc_receiver_impl : ossia::net::udp_receiv
   using udp_receive_socket::udp_receive_socket;
 };
 
-struct http_async_answer
-{
-  std::weak_ptr<oscquery_shared_async_state> state;
-
-  template <typename T, typename S>
-  void operator()(T& req, const S& str)
-  {
-    if(auto ptr = state.lock())
-    {
-      if(ptr->active)
-      {
-        if(ptr->self.on_text_ws_message({}, str))
-          req.close();
-      }
-    }
-  }
-};
-struct http_async_value_answer
-{
-  std::weak_ptr<oscquery_shared_async_state> state;
-  std::string source_address;
-
-  template <typename T, typename S>
-  void operator()(T& req, const S& str)
-  {
-    if(auto ptr = state.lock())
-    {
-      if(ptr->active)
-      {
-        if(ptr->self.on_value_http_message(source_address, str))
-          req.close();
-      }
-    }
-  }
-};
-
-struct http_async_error
-{
-  template <typename T>
-  void operator()(T& req)
-  {
-    req.close();
-  }
-};
-
-template <typename T>
-auto wait_for_future(
-    const std::future<T>& fut, std::chrono::milliseconds timeout,
-    boost::asio::io_context& ctx)
-{
-  std::future_status status = fut.wait_for(std::chrono::seconds(0));
-
-  auto t0 = std::chrono::steady_clock::now();
-
-  while(status != std::future_status::ready)
-  {
-    ctx.poll_one();
-    if((status = fut.wait_for(std::chrono::seconds(0))) == std::future_status::ready)
-      return status;
-
-    auto t1 = std::chrono::steady_clock::now();
-    if(std::chrono::duration_cast<std::chrono::seconds>(t1 - t0) > timeout)
-    {
-      return status;
-      break;
-    }
-  }
-
-  return status;
-}
-using http_async_request
-    = ossia::net::http_get_request<http_async_answer, http_async_error>;
-using http_async_value_request
-    = ossia::net::http_get_request<http_async_value_answer, http_async_error>;
-
-struct http_async_client_context
-{
-  std::shared_ptr<boost::asio::io_service::work> worker;
-};
-
 oscquery_mirror_asio_protocol::oscquery_mirror_asio_protocol(
     ossia::net::network_context_ptr ctx, std::string host, uint16_t local_osc_port)
     : protocol_base{flags{SupportsMultiplex}}
@@ -510,7 +428,7 @@ void oscquery_mirror_asio_protocol::start_websockets()
 void oscquery_mirror_asio_protocol::start_osc()
 {
   m_oscServer = std::make_unique<osc_receiver_impl>(
-      ossia::net::socket_configuration{"0.0.0.0", (uint16_t)m_osc_port},
+      ossia::net::inbound_socket_configuration{"0.0.0.0", (uint16_t)m_osc_port},
       this->m_ctx->context);
   m_oscServer->open();
   m_osc_port = m_oscServer->m_socket.local_endpoint().port();
@@ -558,19 +476,6 @@ void oscquery_mirror_asio_protocol::on_osc_message(const oscpack::ReceivedMessag
       std::chrono::duration_cast<std::chrono::microseconds>(t2 - t1).count());
 #endif
 }
-static std::string asio_to_ip(std::string uri)
-{
-  uri = boost::algorithm::ierase_first_copy(uri, "http://");
-  uri = boost::algorithm::ierase_first_copy(uri, "https://");
-  uri = boost::algorithm::ierase_first_copy(uri, "ws://");
-  uri = boost::algorithm::ierase_first_copy(uri, "wss://");
-
-  auto pos = uri.find_last_of(':');
-  if(pos != std::string::npos)
-    uri.erase(pos, uri.size());
-
-  return uri;
-}
 
 bool oscquery_mirror_asio_protocol::on_binary_ws_message(
     oscquery_mirror_asio_protocol::connection_handler hdl, const std::string& message)
@@ -621,7 +526,7 @@ bool oscquery_mirror_asio_protocol::on_text_ws_message(
             const auto& server_host = asio_to_ip(*m_host_info.osc_ip);
             uint16_t server_port = uint16_t(*m_host_info.osc_port);
             m_oscSender = std::make_unique<osc_sender_impl>(
-                ossia::net::socket_configuration{server_host, server_port},
+                ossia::net::outbound_socket_configuration{server_host, server_port},
                 this->m_ctx->context);
             m_oscSender->connect();
             uint16_t local_server_port = m_oscServer->m_socket.local_endpoint().port();
