@@ -1,41 +1,9 @@
 #pragma once
 
-#include <ossia/detail/json_fwd.hpp>
-#include <ossia/detail/lockfree_queue.hpp>
-#include <ossia/network/base/listening.hpp>
-#include <ossia/network/base/protocol.hpp>
-#include <ossia/network/oscquery/host_info.hpp>
+#include <ossia/protocols/oscquery/oscquery_fwd.hpp>
 
-#include <atomic>
-
-namespace osc
+namespace ossia::oscquery_asio
 {
-template <typename T>
-class sender;
-class receiver;
-}
-
-namespace oscpack
-{
-class ReceivedMessage;
-class IpEndpointName;
-}
-
-namespace ossia
-{
-namespace net
-{
-class websocket_client;
-struct parameter_data;
-struct network_context;
-using network_context_ptr = std::shared_ptr<network_context>;
-}
-namespace oscquery_asio
-{
-struct osc_outbound_visitor;
-struct http_async_client_context;
-struct http_responder;
-
 class oscquery_mirror_asio_protocol;
 struct oscquery_shared_async_state
 {
@@ -47,6 +15,7 @@ class OSSIA_EXPORT oscquery_mirror_asio_protocol final : public ossia::net::prot
 {
   struct osc_sender_impl;
   struct osc_receiver_impl;
+  using async_state = oscquery_shared_async_state;
 
 public:
   oscquery_mirror_asio_protocol(
@@ -121,9 +90,16 @@ public:
 
   void set_feedback(bool fb) override;
 private:
-  friend struct http_async_answer;
-  friend struct http_async_value_answer;
+  friend struct http_async_answer<async_state>;
+  friend struct http_async_value_answer<async_state>;
+  using http_async_request = ossia::oscquery_asio::http_async_request<async_state>;
+  using http_async_value_request
+      = ossia::oscquery_asio::http_async_value_request<async_state>;
+  using http_async_answer = ossia::oscquery_asio::http_async_answer<async_state>;
+  using http_async_value_answer
+      = ossia::oscquery_asio::http_async_value_answer<async_state>;
   using connection_handler = std::weak_ptr<void>;
+
   void on_ws_disconnected() { m_hasWS = false; }
 
   void init();
@@ -160,7 +136,7 @@ private:
   std::unique_ptr<osc_receiver_impl> m_oscServer;
 
   std::unique_ptr<ossia::net::websocket_client> m_websocketClient;
-  std::shared_ptr<oscquery_shared_async_state> m_async_state;
+  std::shared_ptr<async_state> m_async_state;
   std::atomic_bool m_hasWS{};
 
   // Listening status of the local software
@@ -189,4 +165,3 @@ private:
   } m_protocol_to_use{any_protocol};
 };
 }
-}
