@@ -1,4 +1,5 @@
 #pragma once
+#include <ossia/detail/hash_map.hpp>
 #include <ossia/detail/lockfree_queue.hpp>
 #include <ossia/detail/mutex.hpp>
 #include <ossia/network/base/listening.hpp>
@@ -7,8 +8,7 @@
 #include <ossia/network/generic/generic_device.hpp>
 #include <ossia/network/sockets/websocket_reply.hpp>
 #include <ossia/network/zeroconf/zeroconf.hpp>
-
-#include <ossia/detail/hash_map.hpp>
+#include <ossia/protocols/osc/osc_factory.hpp>
 
 #include <nano_signal_slot.hpp>
 
@@ -41,7 +41,7 @@ namespace oscquery_asio
 struct oscquery_client;
 using clients = std::vector<std::unique_ptr<oscquery_client>>;
 //! Implementation of an oscquery server.
-class OSSIA_EXPORT oscquery_server_protocol final : public ossia::net::protocol_base
+class OSSIA_EXPORT oscquery_server_protocol_base : public ossia::net::protocol_base
 {
   friend struct oscquery_client;
   friend class ossia::oscquery::query_answerer;
@@ -50,10 +50,17 @@ class OSSIA_EXPORT oscquery_server_protocol final : public ossia::net::protocol_
 
 public:
   using connection_handler = std::weak_ptr<void>;
-  oscquery_server_protocol(
-      ossia::net::network_context_ptr ctx, uint16_t osc_port = 1234,
-      uint16_t ws_port = 5678, bool forceWS = false);
-  ~oscquery_server_protocol() override;
+
+  // Use oscquery_server_protocol_with_osc
+  explicit oscquery_server_protocol_base(
+      ossia::net::network_context_ptr ctx, uint16_t, uint16_t)
+      = delete;
+
+  oscquery_server_protocol_base(
+      ossia::net::network_context_ptr ctx,
+      const std::vector<ossia::net::osc_server_configuration>& conf, uint16_t ws_port,
+      bool forceWS);
+  ~oscquery_server_protocol_base() override;
 
   bool pull(net::parameter_base&) override;
   std::future<void> pull_async(net::parameter_base&) override;
@@ -71,16 +78,29 @@ public:
 
   void set_device(net::device_base& dev) override;
   void stop() override;
-  ossia::net::device_base& get_device() const { return *m_device; }
+  ossia::net::device_base& get_device() const noexcept { return *m_device; }
 
-  int get_osc_port() const { return m_oscPort; }
+  int get_ws_port() const noexcept { return m_wsPort; }
 
-  int get_ws_port() const { return m_wsPort; }
+  bool force_ws() const noexcept { return m_forceWS.load(std::memory_order_relaxed); }
+  void set_force_ws(bool forceWS) noexcept;
+
+  const std::vector<ossia::net::osc_server_configuration>&
+  get_transports() const noexcept
+  {
+    return m_oscConf;
+  }
 
   Nano::Signal<void(const std::string&)> onClientConnected;
   Nano::Signal<void(const std::string&)> onClientDisconnected;
 
-private:
+protected:
+  explicit oscquery_server_protocol_base(ossia::net::network_context_ptr ctx) = delete;
+
+  // OSC callback
+  void on_osc_message(const oscpack::ReceivedMessage& m);
+  void process_raw_osc_data(const char* data, std::size_t sz);
+
   // List of connected clients
   oscquery_client* find_client(const connection_handler& hdl);
 
@@ -88,10 +108,6 @@ private:
   void remove_node(std::string_view path, const std::string& node);
   void rename_node(std::string_view node, const std::string& new_name);
 
-  // OSC callback
-  void on_osc_message(const oscpack::ReceivedMessage& m);
-  void process_raw_osc_data(const char* data, std::size_t sz);
-
   // Websocket callbacks
   void on_connectionOpen(const connection_handler& hdl);
   void on_connectionClosed(const connection_handler& hdl);
@@ -118,12 +134,12 @@ private:
 
   ossia::net::network_context_ptr m_context;
 
-  struct osc_receiver_impl;
-  std::unique_ptr<osc_receiver_impl> m_oscServer;
   std::unique_ptr<ossia::net::websocket_server> m_websocketServer;
 
   net::zeroconf_server m_zeroconfServerWS;
-  net::zeroconf_server m_zeroconfServerOSC;
+
+  // What we expose over OSC
+  std::vector<ossia::net::osc_server_configuration> m_oscConf;
 
   // Listening status of the local software
   net::listened_parameters m_listening;
@@ -138,11 +154,31 @@ private:
   mutex_t m_clientsMutex;
 
   // The local ports
-  uint16_t m_oscPort{};
   uint16_t m_wsPort{};
 
   // Will only send changes through WS
-  bool m_forceWS{};
+  std::atomic_bool m_forceWS{};
+};
+
+class OSSIA_EXPORT oscquery_server_protocol : public oscquery_server_protocol_base
+{
+public:
+  struct osc_receiver_impl;
+  explicit oscquery_server_protocol(
+      ossia::net::network_context_ptr ctx, uint16_t osc_port = 1234,
+      uint16_t ws_port = 5678, bool forceWS = false);
+  ~oscquery_server_protocol();
+
+  void update_zeroconf();
+  void set_device(net::device_base& dev) override;
+  void stop() override;
+  int get_osc_port() const noexcept { return m_oscPort; }
+
+private:
+  uint16_t m_oscPort{};
+  std::unique_ptr<osc_receiver_impl> m_oscServer;
+
+  net::zeroconf_server m_zeroconfServerOSC;
 };
 }
 }
