@@ -54,28 +54,6 @@
 #define THREADSIGNAL
 #endif
 
-#ifndef HAVE_ALLOCA     /* can work without alloca() but we never need it */
-# define HAVE_ALLOCA 1
-#endif
-#ifdef ALLOCA
-# undef ALLOCA
-#endif
-#ifdef FREEA
-# undef FREEA
-#endif
-
-#if HAVE_ALLOCA
-# define ALLOCA(t, x, n, max) ((x) = (t *)((n) < (max) ?            \
-            alloca((n) * sizeof(t)) : getbytes((n) * sizeof(t))))
-# define FREEA(t, x, n, max) (                                  \
-        ((n) < (max) || (freebytes((x), (n) * sizeof(t)), 0)))
-#else
-# define ALLOCA(t, x, n, max) ((x) = (t *)getbytes((n) * sizeof(t)))
-# define FREEA(t, x, n, max) (freebytes((x), (n) * sizeof(t)))
-#endif
-
-#define MAX_ALLOCA_SAMPLES 16*1024
-
 static jack_nframes_t jack_out_max;
 static jack_nframes_t jack_filled = 0;
 static int jack_started = 0;
@@ -84,8 +62,7 @@ static jack_port_t *output_port[MAX_JACK_PORTS];
 static jack_client_t *jack_client = NULL;
 static char * desired_client_name = NULL;
 char *jack_client_names[MAX_CLIENTS];
-static volatile int jack_dio_error;
-static volatile int jack_didshutdown;
+static int jack_dio_error;
 static t_audiocallback jack_callback;
 static int jack_should_autoconnect = 1;
 static int jack_blocksize = 0; /* should this be PERTHREAD? */
@@ -161,7 +138,6 @@ static int jack_polling_callback(jack_nframes_t nframes, void *unused)
 #ifdef THREADSIGNAL
     sys_semaphore_post(jack_sem);
 #endif
-    FREEA(t_sample, muxbuffer, muxbufsize, MAX_ALLOCA_SAMPLES);
     return 0;
 }
 
@@ -210,13 +186,10 @@ static int callbackprocess(jack_nframes_t nframes, void *arg)
 
 static int jack_srate(jack_nframes_t srate, void *arg)
 {
-    sys_lock();
-    if (srate != STUFF->st_dacsr)
-    {
-        STUFF->st_dacsr = srate;
+    const t_float oldrate = STUFF->st_dacsr;
+    STUFF->st_dacsr = srate;
+    if (oldrate != STUFF->st_dacsr)
         canvas_update_dsp();
-    }
-    sys_unlock();
     return 0;
 }
 
@@ -226,18 +199,17 @@ static int jack_bsize(jack_nframes_t bufsize, void *arg)
     return 0;
 }
 
-    /* This callback function must be async-signal-safe, so the only thing
-    we can really do is set a flag and wake up the scheduler. The shutdown
-    is actually handled in jack_send_dacs() or sys_try_reopen_audio();
-    the latter is called by the callback scheduler once it has noticed that
-    audio processing has halted. */
+void glob_audio_setapi(void *dummy, t_floatarg f);
+
 static void jack_shutdown(void *arg)
 {
-    jack_didshutdown = 1;
-#ifdef THREADSIGNAL
-        /* sem_post() is async-signal-safe */
-    sys_semaphore_post(jack_sem);
-#endif
+    pd_error(0, "JACK: server shut down");
+
+    jack_deactivate (jack_client);
+    jack_client = NULL;
+    jack_blocksize = 0;
+
+    glob_audio_setapi(NULL, API_NONE); // set pd_whichapi 0
 }
 
 static int jack_xrun(void* arg)
@@ -360,9 +332,7 @@ static int jack_connect_ports(char* client)
 
 static void pd_jack_error_callback(const char *desc)
 {
-    sys_lock();
-    logpost(0, PD_DEBUG, "JACK error: %s", desc);
-    sys_unlock();
+    pd_error(0, "JACKerror: %s", desc);
     return;
 }
 
@@ -386,10 +356,10 @@ int jack_open_audio(int inchans, int outchans, t_audiocallback callback)
 #ifdef THREADSIGNAL
     jack_sem = sys_semaphore_create();
 #endif
-    jack_didshutdown = 0;
+
     jack_dio_error = 0;
 
-    if ((inchans == 0) && (outchans == 0)) return 1;
+    if ((inchans == 0) && (outchans == 0)) return 0;
 
     if (outchans > MAX_JACK_PORTS) {
         pd_error(0, "JACK: %d output ports not supported, setting to %d",
@@ -474,7 +444,7 @@ int jack_open_audio(int inchans, int outchans, t_audiocallback callback)
     advance_samples = sys_schedadvance * (double)STUFF->st_dacsr / 1.e6;
     advance_samples -= (advance_samples % DEFDACBLKSIZE);
         /* make sure that the delay is not smaller than the Jack blocksize! */
-    if (!callback && advance_samples < jack_blocksize)
+    if (advance_samples < jack_blocksize)
     {
         int delay = ((double)sys_schedadvance / 1000.) + 0.5;
         int limit = ceil(jack_blocksize * 1000. / (double)STUFF->st_dacsr);
@@ -572,20 +542,6 @@ void jack_close_audio(void)
 #endif
 }
 
-void sys_do_close_audio(void);
-
-int jack_reopen_audio(void)
-{
-        /* we don't actually try to reopen (yet?) */
-    if (jack_didshutdown)
-    {
-        pd_error(0, "JACK: server shutdown");
-        jack_didshutdown = 0;
-    }
-    sys_do_close_audio();
-    return 0;
-}
-
 int sched_idletask(void);
 
 int jack_send_dacs(void)
@@ -607,11 +563,6 @@ int jack_send_dacs(void)
         (sys_ringbuf_getwriteavailable(&jack_outring) <
             (long)(STUFF->st_outchannels * DEFDACBLKSIZE*sizeof(t_sample))))
     {
-        if (jack_didshutdown)
-        {
-            jack_reopen_audio(); /* handle server shutdown */
-            return (SENDDACS_NO);
-        }
 #ifdef THREADSIGNAL
         if (sched_idletask())
             continue;
