@@ -64,7 +64,9 @@ Distributed under the Boost Software License, Version 1.0.
 #include "execinfo_win64.h"
 #else
 #include <dlfcn.h>
+#if !QUICKCPPLIB_DISABLE_EXECINFO
 #include <execinfo.h>
+#endif
 #include <fcntl.h>
 #include <limits.h>
 #include <signal.h>  // for siginfo_t
@@ -149,7 +151,11 @@ namespace ringbuffer_log
           , code32{_code1, _code2}
           , level(static_cast<uint8_t>(_level))
           , using_code64(false)
+#if !QUICKCPPLIB_DISABLE_EXECINFO
           , using_backtrace(!_function)
+#else
+          , using_backtrace{0}
+#endif
       {
         size_t _messagelen = strlen(_message) + 1;
         if(_messagelen > sizeof(message))
@@ -191,10 +197,12 @@ namespace ringbuffer_log
         else
         {
           function[0] = 0;
+#if !QUICKCPPLIB_DISABLE_EXECINFO
           const void *temp[16];
           memset(temp, 0, sizeof(temp));
           (void) ::backtrace((void **) temp, 16);
           packed_backtrace::make_packed_backtrace(backtrace, temp);
+#endif
         }
       }
       bool operator==(const value_type &o) const noexcept { return memcmp(this, &o, sizeof(*this)) == 0; }
@@ -301,13 +309,15 @@ namespace ringbuffer_log
                               diff = (ssize_t) backtrace[n];
                             }
                             char buffer[32];
-                            sprintf(buffer, "%zx", diff);
+                            snprintf(buffer, 32, "%zx", diff);
+                            buffer[31] = 0;
                             addrs.append(buffer);
                           }
                           else
                           {
                             char buffer[32];
-                            sprintf(buffer, "%zx", (uintptr_t) backtrace[n]);
+                            snprintf(buffer, 32, "%zx", (uintptr_t) backtrace[n]);
+                            buffer[31] = 0;
                             addrs.append(buffer);
                           }
                           addrs.push_back('\n');
@@ -382,7 +392,9 @@ namespace ringbuffer_log
         }
         if(!done)
 #endif
+
         {
+#if !QUICKCPPLIB_DISABLE_EXECINFO
           char **symbols = backtrace_symbols(backtrace, len);
           if(!symbols)
             ret.append("BACKTRACE FAILED!");
@@ -399,6 +411,7 @@ namespace ringbuffer_log
             }
             free(symbols);
           }
+#endif
         }
       }
       else
@@ -467,6 +480,7 @@ namespace ringbuffer_log
       s << temp << "\",\"";
       if(v.using_backtrace)
       {
+#if !QUICKCPPLIB_DISABLE_EXECINFO
         char **symbols = backtrace_symbols((void **) v.backtrace, sizeof(v.backtrace) / sizeof(v.backtrace[0]));
         if(!symbols)
           s << "BACKTRACE FAILED!";
@@ -483,6 +497,7 @@ namespace ringbuffer_log
           }
           free(symbols);
         }
+#endif
       }
       else
       {
