@@ -39,6 +39,21 @@ if [[ ! -d score-addon-ble ]]; then
   git clone --recursive -j16 https://github.com/ossia/score-addon-ble
 fi
 
+if [[ ! -d score-addon-led ]]; then
+  git clone --recursive -j16 https://github.com/ossia/score-addon-led
+fi
+
+if [[ ! -d score-addon-spatgris ]]; then
+  git clone --recursive -j16 https://github.com/ossia/score-addon-spatgris
+fi
+
+if [[ ! -d score-addon-hdf5 ]]; then
+  git clone --recursive -j16 https://github.com/ossia/score-addon-hdf5
+fi
+
+if [[ ! -d score-addon-puara ]]; then
+  git clone --recursive -j16 https://github.com/ossia/score-addon-puara
+fi
 
 )
 
