@@ -24,7 +24,11 @@ if [[ "${CI_IS_AZURE}" = "1" ]]; then
   security unlock-keychain -p travis build.keychain
   export PACKAGE_ARCH=Intel
 else
-  export PACKAGE_ARCH=AppleSilicon
+  if [[ "$MACOS_ARCH" = "x86_64" ]]; then
+    export PACKAGE_ARCH=Intel
+  else
+    export PACKAGE_ARCH=AppleSilicon
+  fi
 fi
 
 
@@ -56,6 +60,7 @@ sudo create-dmg \
   --app-drop-link 600 185 \
   --icon "ossia score.app" 200 190 \
   --hide-extension "ossia score.app" \
+  --filesystem APFS \
   'score.dmg' 'ossia score.app'
 
 sudo chown "$(whoami)" ./*.dmg
@@ -81,4 +86,4 @@ xcrun stapler validate ./*.dmg
 
 # Archive
 mv ./*.dmg "$BUILD_ARTIFACTSTAGINGDIRECTORY/ossia score-$TAG-macOS-$PACKAGE_ARCH.dmg"
-mv "mac-sdk.zip" "$BUILD_ARTIFACTSTAGINGDIRECTORY/"
+mv "mac-sdk.zip" "$BUILD_ARTIFACTSTAGINGDIRECTORY/mac-sdk-$PACKAGE_ARCH.zip"
