@@ -170,13 +170,25 @@ namespace
 {
 bool runningUnderAnUISession() noexcept
 {
-#if __linux__
-  return !qgetenv("DISPLAY").isEmpty() || !qgetenv("WAYLAND_DISPLAY").isEmpty()
-         || (qgetenv("XDG_SESSION_TYPE") != "tty")
-         || (qgetenv("QT_QPA_PLATFORM").contains("gl"));
-#else
+  const auto platform = qgetenv("QT_QPA_PLATFORM");
+  if(platform == "minimal")
+  {
+    return false;
+  }
+
   // Win32 and macOS always have a graphical session
+#if defined(WIN32) || defined(__APPLE__)
   return true;
+#else
+  if(!qEnvironmentVariableIsEmpty("DISPLAY"))
+    return true;
+  if(!qEnvironmentVariableIsEmpty("WAYLAND_DISPLAY"))
+    return true;
+  if(qgetenv("XDG_SESSION_TYPE") != "tty")
+    return true;
+  if(platform.contains("gl"))
+    return true;
+  return false;
 #endif
 }
 
@@ -414,28 +426,50 @@ void Application::init()
   {
     m_view->sizeChanged(m_view->size());
     m_view->ready();
+
+    auto sqa = safe_cast<SafeQApplication*>(m_app);
+    connect(sqa, &SafeQApplication::fileOpened, this, [&](const QString& file) {
+      auto& ctx = m_presenter->applicationContext();
+      m_presenter->documentManager().loadFile(ctx, file);
+    });
   }
 
-  QTimer::singleShot(10, [&] { initDocuments(); });
+  QTimer::singleShot(10, [&] {
+    initDocuments();
+
+#if defined(QT_FEATURE_thread)
+#if QT_FEATURE_thread == 1
+    QThreadPool::globalInstance()->setMaxThreadCount(2);
+#endif
+#endif
+
+    auto& ctx = m_presenter->applicationContext();
+    // The plug-ins have the ability to override the boot process.
+    for(auto plug : ctx.guiApplicationPlugins())
+    {
+      plug->afterStartup();
+    }
+  });
 }
 
 void Application::initDocuments()
 {
   auto& ctx = m_presenter->applicationContext();
+  // The plug-ins have the ability to override the boot process.
+  for(auto plug : ctx.guiApplicationPlugins())
+  {
+    if(plug->handleLoading())
+    {
+      return;
+    }
+  }
+
   if(!appSettings.loadList.empty())
   {
     for(const auto& doc : appSettings.loadList)
       m_presenter->documentManager().loadFile(ctx, doc);
   }
 
-  if(appSettings.gui)
-  {
-    auto sqa = safe_cast<SafeQApplication*>(m_app);
-    connect(sqa, &SafeQApplication::fileOpened, this, [&](const QString& file) {
-      m_presenter->documentManager().loadFile(ctx, file);
-    });
-  }
-
   // Try to reload if there was a crash
   if(appSettings.forceRestore)
   {
@@ -467,21 +501,6 @@ void Application::initDocuments()
 #if !defined(SCORE_SPLASH_SCREEN)
   openNewDocument();
 #endif
-
-#if defined(QT_FEATURE_thread)
-#if QT_FEATURE_thread == 1
-  QThreadPool::globalInstance()->setMaxThreadCount(2);
-#endif
-#endif
-
-  // The plug-ins have the ability to override the boot process.
-  for(auto plug : ctx.guiApplicationPlugins())
-  {
-    if(plug->handleStartup())
-    {
-      return;
-    }
-  }
 }
 
 void Application::openNewDocument()
