@@ -10,11 +10,8 @@
 #include <score/document/DocumentContext.hpp>
 #include <score/graphics/GraphicWidgets.hpp>
 #include <score/graphics/GraphicsItem.hpp>
+#include <score/graphics/RectItem.hpp>
 #include <score/graphics/TextItem.hpp>
-#include <score/graphics/widgets/QGraphicsMultiSlider.hpp>
-#include <score/graphics/widgets/QGraphicsTextButton.hpp>
-#include <score/graphics/widgets/QGraphicsXYSpinbox.hpp>
-#include <score/graphics/widgets/QGraphicsXYZSpinbox.hpp>
 #include <score/tools/Unused.hpp>
 #include <score/widgets/ComboBox.hpp>
 #include <score/widgets/ControlWidgets.hpp>
@@ -24,6 +21,10 @@
 #include <ossia/network/domain/domain_functions.hpp>
 #include <ossia/network/value/value_conversion.hpp>
 
+#include <ossia-qt/invoke.hpp>
+
+#include <QAction>
+#include <QApplication>
 #include <QCheckBox>
 #include <QFileDialog>
 #include <QGraphicsItem>
@@ -32,8 +33,6 @@
 #include <QPalette>
 #include <QTextDocument>
 
-#include <private/qwidgettextcontrol_p.h>
-
 #include <score_lib_process_export.h>
 
 #include <verdigris>
@@ -54,35 +53,6 @@ struct SCORE_LIB_PROCESS_EXPORT DefaultControlLayouts
   static Process::PortItemLayout button() noexcept;
   static Process::PortItemLayout chooser_toggle() noexcept;
 };
-
-struct SCORE_LIB_PROCESS_EXPORT LineEditItem : public QGraphicsTextItem
-{
-  W_OBJECT(LineEditItem)
-public:
-  explicit LineEditItem(QGraphicsItem* parent)
-      : QGraphicsTextItem{parent}
-  {
-    setTextInteractionFlags(Qt::TextEditorInteraction);
-    auto ctl = this->findChild<QWidgetTextControl*>();
-    if(ctl)
-    {
-      ctl->setAcceptRichText(false);
-    }
-  }
-
-  void dropEvent(QGraphicsSceneDragDropEvent* drop) override
-  {
-    QGraphicsItem::dropEvent(drop);
-    const auto& urlList = drop->mimeData()->urls();
-
-    if(!urlList.isEmpty())
-    {
-      this->setPlainText(urlList[0].toLocalFile());
-    }
-  }
-
-  void sizeChanged(QSizeF sz) E_SIGNAL(SCORE_LIB_PROCESS_EXPORT, sizeChanged, sz)
-};
 }
 
 namespace WidgetFactory
@@ -116,17 +86,16 @@ struct FloatControl
     }
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
-    ConcreteNormalizer<Normalizer, T> norm{slider};
+    ConcreteNormalizer<Normalizer, T> norm{inlet};
 
     auto sl = new score::ValueDoubleSlider{parent};
     sl->setOrientation(Qt::Horizontal);
     sl->setContentsMargins(0, 0, 0, 0);
-    bindFloatDomain(slider, inlet, *sl);
+    bindFloatDomain(inlet, inlet, *sl);
     sl->setValue(norm.to01(ossia::convert<double>(inlet.value())));
 
     if constexpr(Control)
@@ -135,30 +104,27 @@ struct FloatControl
           sl, &score::DoubleSlider::sliderMoved, context,
           [sl, norm, &inlet, &ctx](double v) {
         sl->moving = true;
-        ctx.dispatcher.submit<SetControlValue<Control_T>>(
-            inlet, norm.from01(sl->value()));
+        ctx.dispatcher.submit<SetControlValue<T>>(inlet, norm.from01(sl->value()));
           });
       QObject::connect(
           sl, &score::DoubleSlider::sliderReleased, context, [sl, norm, &inlet, &ctx]() {
-            ctx.dispatcher.submit<SetControlValue<Control_T>>(
-                inlet, norm.from01(sl->value()));
-            ctx.dispatcher.commit();
-            sl->moving = false;
-          });
+        ctx.dispatcher.submit<SetControlValue<T>>(inlet, norm.from01(sl->value()));
+        ctx.dispatcher.commit();
+        sl->moving = false;
+      });
     }
 
-    QObject::connect(
-        &inlet, &Control_T::valueChanged, sl, [sl, norm](const ossia::value& val) {
-          if constexpr(Control)
-          {
-            if(!sl->moving)
-              sl->setValue(norm.to01(ossia::convert<double>(val)));
-          }
-          else
-          {
-            sl->setValue(norm.to01(ossia::convert<double>(val)));
-          }
-        });
+    QObject::connect(&inlet, &T::valueChanged, sl, [sl, norm](const ossia::value& val) {
+      if constexpr(Control)
+      {
+        if(!sl->moving)
+          sl->setValue(norm.to01(ossia::convert<double>(val)));
+      }
+      else
+      {
+        sl->setValue(norm.to01(ossia::convert<double>(val)));
+      }
+    });
 
     return sl;
   }
@@ -171,6 +137,7 @@ struct FloatControl
     ConcreteNormalizer<Normalizer, T> norm{slider};
 
     auto sl = new ControlUI{nullptr};
+    initWidgetProperties(inlet, *sl);
     bindFloatDomain(slider, inlet, *sl);
     sl->setValue(norm.to01(ossia::convert<double>(inlet.value())));
 
@@ -227,33 +194,31 @@ struct IntSlider
     return Process::DefaultControlLayouts::slider();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto sl = new score::ValueSlider{parent};
     sl->setOrientation(Qt::Horizontal);
-    bindIntDomain(slider, inlet, *sl);
+    bindIntDomain(inlet, inlet, *sl);
     sl->setValue(ossia::convert<int>(inlet.value()));
     sl->setContentsMargins(0, 0, 0, 0);
 
     QObject::connect(
         sl, &score::IntSlider::sliderMoved, context, [sl, &inlet, &ctx](int p) {
           sl->moving = true;
-          ctx.dispatcher.submit<SetControlValue<Control_T>>(inlet, p);
+          ctx.dispatcher.submit<SetControlValue<T>>(inlet, p);
         });
     QObject::connect(sl, &score::IntSlider::sliderReleased, context, [sl, &inlet, &ctx] {
-      ctx.dispatcher.submit<SetControlValue<Control_T>>(inlet, sl->value());
+      ctx.dispatcher.submit<SetControlValue<T>>(inlet, sl->value());
       ctx.dispatcher.commit();
       sl->moving = false;
     });
 
-    QObject::connect(
-        &inlet, &Control_T::valueChanged, sl, [sl](const ossia::value& val) {
-          if(!sl->moving)
-            sl->setValue(ossia::convert<int>(val));
-        });
+    QObject::connect(&inlet, &T::valueChanged, sl, [sl](const ossia::value& val) {
+      if(!sl->moving)
+        sl->setValue(ossia::convert<int>(val));
+    });
 
     return sl;
   }
@@ -264,6 +229,7 @@ struct IntSlider
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsIntSlider{nullptr};
+    initWidgetProperties(inlet, *sl);
     bindIntDomain(slider, inlet, *sl);
     sl->setValue(ossia::convert<int>(inlet.value()));
 
@@ -301,10 +267,9 @@ struct IntRangeSlider
     return Process::DefaultControlLayouts::slider();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     // TODO
     return nullptr;
@@ -316,6 +281,7 @@ struct IntRangeSlider
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsRangeSlider{parent};
+    initWidgetProperties(inlet, *sl);
     bindIntDomain(slider, inlet, *sl);
     sl->setValue(ossia::convert<ossia::vec2f>(inlet.value()));
 
@@ -353,10 +319,9 @@ struct FloatRangeSlider
     return Process::DefaultControlLayouts::slider();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     // TODO
     return nullptr;
@@ -368,6 +333,7 @@ struct FloatRangeSlider
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsRangeSlider{parent};
+    initWidgetProperties(inlet, *sl);
     bindFloatDomain(slider, inlet, *sl);
     sl->setValue(ossia::convert<ossia::vec2f>(inlet.value()));
 
@@ -405,10 +371,9 @@ struct FloatRangeSpinBox
     return Process::DefaultControlLayouts::pad();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     SCORE_TODO;
     return nullptr; // TODO
@@ -420,6 +385,7 @@ struct FloatRangeSpinBox
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsXYSpinboxChooser{true, nullptr};
+    initWidgetProperties(inlet, *sl);
     bindVec2Domain(slider, inlet, *sl);
     sl->setValue(
         LinearNormalizer::to01(*sl, ossia::convert<ossia::vec2f>(inlet.value())));
@@ -452,25 +418,23 @@ struct IntSpinBox
     return Process::DefaultControlLayouts::spinbox();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto sl = new QSpinBox{parent};
-    bindIntDomain(slider, inlet, *sl);
+    bindIntDomain(inlet, inlet, *sl);
     sl->setValue(ossia::convert<int>(inlet.value()));
     sl->setContentsMargins(0, 0, 0, 0);
 
     QObject::connect(
         sl, SignalUtils::QSpinBox_valueChanged_int(), context, [&inlet, &ctx](int val) {
-          CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
-              inlet, val);
-        });
+      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<T>>(inlet, val);
+    });
 
-    QObject::connect(
-        &inlet, &Control_T::valueChanged, sl,
-        [sl](const ossia::value& val) { sl->setValue(ossia::convert<int>(val)); });
+    QObject::connect(&inlet, &T::valueChanged, sl, [sl](const ossia::value& val) {
+      sl->setValue(ossia::convert<int>(val));
+    });
 
     return sl;
   }
@@ -481,6 +445,7 @@ struct IntSpinBox
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsIntSpinbox{nullptr};
+    initWidgetProperties(inlet, *sl);
     bindIntDomain(slider, inlet, *sl);
     sl->setValue(ossia::convert<int>(inlet.value()));
 
@@ -520,26 +485,24 @@ struct FloatSpinBox
     return Process::DefaultControlLayouts::spinbox();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto sl = new QDoubleSpinBox{parent};
-    bindFloatDomain(slider, inlet, *sl);
+    bindFloatDomain(inlet, inlet, *sl);
     sl->setValue(ossia::convert<float>(inlet.value()));
     sl->setContentsMargins(0, 0, 0, 0);
 
     QObject::connect(
         sl, SignalUtils::QDoubleSpinBox_valueChanged_double(), context,
         [&inlet, &ctx](double val) {
-      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
-          inlet, val);
-        });
+      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<T>>(inlet, val);
+    });
 
-    QObject::connect(
-        &inlet, &Control_T::valueChanged, sl,
-        [sl](const ossia::value& val) { sl->setValue(ossia::convert<float>(val)); });
+    QObject::connect(&inlet, &T::valueChanged, sl, [sl](const ossia::value& val) {
+      sl->setValue(ossia::convert<float>(val));
+    });
 
     return sl;
   }
@@ -552,6 +515,7 @@ struct FloatSpinBox
     ConcreteNormalizer<LinearNormalizer, T> norm{slider};
 
     auto sl = new score::QGraphicsSpinbox{nullptr};
+    initWidgetProperties(inlet, *sl);
     bindFloatDomain(slider, inlet, *sl);
     sl->setValue(norm.to01(ossia::convert<float>(inlet.value())));
 
@@ -593,10 +557,9 @@ struct TimeChooser
     return DefaultControlLayouts::knob();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     // FIXME
     return nullptr;
@@ -608,6 +571,7 @@ struct TimeChooser
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsTimeChooser{nullptr};
+    initWidgetProperties(inlet, *sl);
     // bindFloatDomain(slider, inlet, *sl);
     sl->setValue(ossia::convert<ossia::vec2f>(inlet.value()));
 
@@ -651,21 +615,19 @@ struct Toggle
     return Process::DefaultControlLayouts::toggle();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& toggle, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto sl = new QCheckBox{parent};
     sl->setChecked(ossia::convert<bool>(inlet.value()));
     QObject::connect(sl, &QCheckBox::toggled, context, [&inlet, &ctx](bool val) {
-      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
-          inlet, val);
+      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<T>>(inlet, val);
     });
 
-    QObject::connect(
-        &inlet, &Control_T::valueChanged, sl,
-        [sl](const ossia::value& val) { sl->setChecked(ossia::convert<bool>(val)); });
+    QObject::connect(&inlet, &T::valueChanged, sl, [sl](const ossia::value& val) {
+      sl->setChecked(ossia::convert<bool>(val));
+    });
 
     return sl;
   }
@@ -676,6 +638,7 @@ struct Toggle
       QGraphicsItem* parent, QObject* context)
   {
     auto cb = new score::QGraphicsCheckBox{nullptr};
+    initWidgetProperties(inlet, *cb);
     cb->setState(ossia::convert<bool>(inlet.value()));
 
     QObject::connect(
@@ -700,17 +663,16 @@ struct ImpulseButton
     return Process::DefaultControlLayouts::bang();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, const Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto sl = new QPushButton{parent};
     const auto& name = inlet.visualName();
     sl->setText(name.isEmpty() ? QObject::tr("Bang") : name);
     sl->setContentsMargins(0, 0, 0, 0);
 
-    auto& cinlet = const_cast<Control_T&>(inlet);
+    auto& cinlet = const_cast<T&>(inlet);
     QObject::connect(sl, &QPushButton::pressed, context, [&cinlet] {
       cinlet.valueChanged(ossia::impulse{});
     });
@@ -724,6 +686,7 @@ struct ImpulseButton
       QGraphicsItem* parent, QObject* context)
   {
     auto toggle = new score::QGraphicsButton{nullptr};
+    initWidgetProperties(inlet, *toggle);
 
     QObject::connect(
         toggle, &score::QGraphicsButton::pressed, context, [=, &inlet](bool pressed) {
@@ -744,10 +707,9 @@ struct Button
     return Process::DefaultControlLayouts::bang();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, const Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto sl = new QPushButton{parent};
     const auto& name = inlet.visualName();
@@ -755,7 +717,7 @@ struct Button
     sl->setContentsMargins(0, 0, 0, 0);
 
     // TODO should we not make a command here
-    auto& cinlet = const_cast<Control_T&>(inlet);
+    auto& cinlet = const_cast<T&>(inlet);
     QObject::connect(
         sl, &QPushButton::pressed, context, [&cinlet] { cinlet.setValue(true); });
     QObject::connect(
@@ -770,6 +732,7 @@ struct Button
       QGraphicsItem* parent, QObject* context)
   {
     auto toggle = new score::QGraphicsButton{nullptr};
+    initWidgetProperties(inlet, *toggle);
 
     QObject::connect(
         toggle, &score::QGraphicsButton::pressed, context,
@@ -801,13 +764,13 @@ struct ChooserToggle
       return t.alternatives;
     }
   }
-  template <typename T, typename Control_T>
+
+  template <typename T>
   static auto make_widget(
-      const T& control, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
 
-    const auto& alts = getAlternatives(control);
+    const auto& alts = getAlternatives(inlet);
     SCORE_ASSERT(alts.size() == 2);
     auto toggleBtn = new score::ToggleButton{alts, parent};
     toggleBtn->setCheckable(true);
@@ -819,19 +782,17 @@ struct ChooserToggle
 
     QObject::connect(
         toggleBtn, &score::ToggleButton::toggled, context, [&inlet, &ctx](bool val) {
-          CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
-              inlet, val);
-        });
+      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<T>>(inlet, val);
+    });
 
     QObject::connect(
-        &inlet, &Control_T::valueChanged, toggleBtn,
-        [toggleBtn](const ossia::value& val) {
+        &inlet, &T::valueChanged, toggleBtn, [toggleBtn](const ossia::value& val) {
       bool b = ossia::convert<bool>(val);
       if(b && !toggleBtn->isChecked())
         toggleBtn->toggle();
       else if(!b && toggleBtn->isChecked())
         toggleBtn->toggle();
-        });
+    });
 
     return toggleBtn;
   }
@@ -844,6 +805,7 @@ struct ChooserToggle
     const auto& alts = getAlternatives(control);
     SCORE_ASSERT(alts.size() == 2);
     auto toggle = new score::QGraphicsToggle{alts[0], alts[1], nullptr};
+    initWidgetProperties(inlet, *toggle);
     toggle->setState(ossia::convert<bool>(inlet.value()));
 
     QObject::connect(
@@ -869,34 +831,33 @@ struct LineEdit
     return Process::DefaultControlLayouts::lineedit();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto sl = new QLineEdit{parent};
     sl->setText(QString::fromStdString(ossia::convert<std::string>(inlet.value())));
     sl->setContentsMargins(0, 0, 0, 0);
     sl->setMaximumWidth(70);
     QObject::connect(sl, &QLineEdit::editingFinished, context, [sl, &inlet, &ctx]() {
-      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
+      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<T>>(
           inlet, sl->text().toStdString());
     });
 
-    QObject::connect(
-        &inlet, &Control_T::valueChanged, sl, [sl](const ossia::value& val) {
-          sl->setText(QString::fromStdString(ossia::convert<std::string>(val)));
-        });
+    QObject::connect(&inlet, &T::valueChanged, sl, [sl](const ossia::value& val) {
+      sl->setText(QString::fromStdString(ossia::convert<std::string>(val)));
+    });
 
     return sl;
   }
 
   template <typename T, typename Control_T>
-  static Process::LineEditItem* make_item(
+  static score::QGraphicsLineEdit* make_item(
       const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
       QGraphicsItem* parent, QObject* context)
   {
-    auto sl = new Process::LineEditItem{parent};
+    auto sl = new score::QGraphicsLineEdit{parent};
+    initWidgetProperties(inlet, *sl);
     sl->setTextWidth(180.);
     sl->setDefaultTextColor(QColor{"#E0B01E"});
     sl->setCursor(Qt::IBeamCursor);
@@ -904,22 +865,31 @@ struct LineEdit
     sl->setPlainText(QString::fromStdString(ossia::convert<std::string>(inlet.value())));
 
     auto doc = sl->document();
-    QObject::connect(
-        doc, &QTextDocument::contentsChanged, context,
-        [=, &inlet, &ctx, r = sl->boundingRect()]() mutable {
+    auto on_edit = [=, &inlet, &ctx] {
       auto cur_str = ossia::convert<std::string>(inlet.value());
       if(cur_str != doc->toPlainText().toStdString())
       {
         CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
             inlet, doc->toPlainText().toStdString());
       }
+    };
+
+    if(!inlet.noValueChangeOnMove)
+    {
+      QObject::connect(doc, &QTextDocument::contentsChanged, context, on_edit);
+    }
+    else
+    {
+      QObject::connect(sl, &score::QGraphicsLineEdit::editingFinished, context, on_edit);
+    }
+
+    if(auto obj = dynamic_cast<score::ResizeableItem*>(context))
+    {
+      QObject::connect(
+          sl, &score::QGraphicsLineEdit::sizeChanged, obj,
+          &score::ResizeableItem::childrenSizeChanged);
+    }
 
-      if(r != sl->boundingRect())
-      {
-        r = sl->boundingRect();
-        sl->sizeChanged(r.size());
-      }
-        });
     QObject::connect(&inlet, &Control_T::valueChanged, sl, [=](const ossia::value& val) {
       auto str = QString::fromStdString(ossia::convert<std::string>(val));
       if(str != doc->toPlainText())
@@ -950,8 +920,13 @@ struct ProgramPortScriptDialog : Process::ScriptDialog
     auto next = this->text().toStdString();
     if(next != cur)
     {
-      CommandDispatcher<>{m_context.commandStack}.submit(
-          new Process::SetControlValue{m_port, next});
+      ossia::qt::run_async(
+          qApp, [&ctx = m_context.commandStack, port = QPointer{&m_port}, next] {
+        if(port)
+        {
+          CommandDispatcher<>{ctx}.submit(new Process::SetControlValue{*port, next});
+        }
+      });
     }
   }
 
@@ -965,10 +940,9 @@ struct ProgramEdit
     return Process::DefaultControlLayouts::lineedit();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto toggle = new QPushButton{QObject::tr("Edit..."), parent};
     QObject::connect(toggle, &QPushButton::clicked, context, [=, &inlet, &ctx] {
@@ -983,10 +957,11 @@ struct ProgramEdit
       QGraphicsItem* parent, QObject* context)
   {
     auto toggle = new score::QGraphicsTextButton{QObject::tr("Edit..."), nullptr};
+    initWidgetProperties(inlet, *toggle);
 
     QObject::connect(
         toggle, &score::QGraphicsTextButton::pressed, context,
-        [=, &inlet, &ctx] { createDialog(inlet, ctx); });
+        [=, &inlet, &ctx] { createDialog(inlet, ctx); }, Qt::QueuedConnection);
 
     return toggle;
   }
@@ -1005,10 +980,9 @@ struct FileChooser
   {
     return Process::DefaultControlLayouts::lineedit();
   }
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     auto sl = new QLineEdit{parent};
     auto act = new QAction{sl};
@@ -1033,13 +1007,12 @@ struct FileChooser
     //sl->setMaximumWidth(70);
 
     QObject::connect(sl, &QLineEdit::editingFinished, context, [sl, &inlet, &ctx]() {
-      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
+      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<T>>(
           inlet, sl->text().toStdString());
     });
-    QObject::connect(
-        &inlet, &Control_T::valueChanged, sl, [sl](const ossia::value& val) {
-          sl->setText(QString::fromStdString(ossia::convert<std::string>(val)));
-        });
+    QObject::connect(&inlet, &T::valueChanged, sl, [sl](const ossia::value& val) {
+      sl->setText(QString::fromStdString(ossia::convert<std::string>(val)));
+    });
     return sl;
   }
 
@@ -1049,6 +1022,7 @@ struct FileChooser
       QGraphicsItem* parent, QObject* context)
   {
     auto bt = new score::QGraphicsTextButton{"Choose a file...", parent};
+    initWidgetProperties(inlet, *bt);
     auto on_open = [&inlet, &ctx] {
       auto filename
           = QFileDialog::getOpenFileName(nullptr, "Open File", {}, inlet.filters());
@@ -1065,8 +1039,10 @@ struct FileChooser
       CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
           inlet, filename.toStdString());
     };
-    QObject::connect(bt, &score::QGraphicsTextButton::pressed, &inlet, on_open);
-    QObject::connect(bt, &score::QGraphicsTextButton::dropped, &inlet, on_set);
+    QObject::connect(
+        bt, &score::QGraphicsTextButton::pressed, &inlet, on_open, Qt::QueuedConnection);
+    QObject::connect(
+        bt, &score::QGraphicsTextButton::dropped, &inlet, on_set, Qt::QueuedConnection);
     auto set = [=](const ossia::value& val) {
       auto str = QString::fromStdString(ossia::convert<std::string>(val));
       if(str != bt->text())
@@ -1101,13 +1077,11 @@ struct Enum
   {
     return QString::fromStdString(str);
   }
-
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
-    const auto& values = slider.getValues();
+    const auto& values = inlet.getValues();
     using val_t = std::remove_reference_t<decltype(values[0])>;
     auto sl = new QComboBox{parent};
     for(const auto& e : values)
@@ -1128,14 +1102,13 @@ struct Enum
     QObject::connect(
         sl, SignalUtils::QComboBox_currentIndexChanged_int(), context,
         [values, &inlet, &ctx](int idx) {
-      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
+      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<T>>(
           inlet, toStd(values[idx]));
-        });
-
-    QObject::connect(&inlet, &Control_T::valueChanged, sl, [=](const ossia::value& val) {
-      set_index(val);
     });
 
+    QObject::connect(
+        &inlet, &T::valueChanged, sl, [=](const ossia::value& val) { set_index(val); });
+
     return sl;
   }
 
@@ -1152,6 +1125,8 @@ struct Enum
                   : (score::QGraphicsEnum*)new score::QGraphicsPixmapEnum{
                       values, slider.pixmaps, nullptr};
 
+    initWidgetProperties(inlet, *sl);
+
     auto set_index = [values, sl](const ossia::value& val) {
       auto v = ossia::convert<std::string>(val);
       auto it = ossia::find(values, convert(v, val_t{}));
@@ -1185,12 +1160,11 @@ struct ComboBox
     return Process::DefaultControlLayouts::combo();
   }
 
-  template <typename U, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const U& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
-    const auto& values = slider.getValues();
+    const auto& values = inlet.getValues();
     auto sl = new score::ComboBox{parent};
     for(auto& e : values)
     {
@@ -1211,14 +1185,13 @@ struct ComboBox
     QObject::connect(
         sl, SignalUtils::QComboBox_currentIndexChanged_int(), context,
         [values, &inlet, &ctx](int idx) {
-      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<Control_T>>(
+      CommandDispatcher<>{ctx.commandStack}.submit<SetControlValue<T>>(
           inlet, values[idx].second);
-        });
-
-    QObject::connect(&inlet, &Control_T::valueChanged, sl, [=](const ossia::value& val) {
-      set_index(val);
     });
 
+    QObject::connect(
+        &inlet, &T::valueChanged, sl, [=](const ossia::value& val) { set_index(val); });
+
     return sl;
   }
 
@@ -1236,6 +1209,7 @@ struct ComboBox
       arr.push_back(values[i].first);
 
     auto sl = new score::QGraphicsCombo{arr, nullptr};
+    initWidgetProperties(inlet, *sl);
 
     auto set_index = [values, sl](const ossia::value& val) {
       auto it
@@ -1295,10 +1269,9 @@ struct HSVSlider
     return Process::DefaultControlLayouts::pad();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     SCORE_TODO;
     return nullptr; // TODO
@@ -1310,6 +1283,7 @@ struct HSVSlider
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsHSVChooser{nullptr};
+    initWidgetProperties(inlet, *sl);
     sl->setRgbaValue(ossia::convert<ossia::vec4f>(inlet.value()));
 
     QObject::connect(
@@ -1340,10 +1314,9 @@ struct XYSlider
     return PortItemLayout{};
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     SCORE_TODO;
     return nullptr; // TODO
@@ -1355,6 +1328,7 @@ struct XYSlider
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsXYChooser{nullptr};
+    initWidgetProperties(inlet, *sl);
     sl->setValue(ossia::convert<ossia::vec2f>(inlet.value()));
     bindVec2Domain(slider, inlet, *sl);
 
@@ -1385,10 +1359,9 @@ struct XYZSlider
     return Process::DefaultControlLayouts::pad();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     SCORE_TODO;
     return nullptr; // TODO
@@ -1400,6 +1373,7 @@ struct XYZSlider
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsXYZChooser{nullptr};
+    initWidgetProperties(inlet, *sl);
     sl->setValue(ossia::convert<ossia::vec3f>(inlet.value()));
     bindVec3Domain(slider, inlet, *sl);
 
@@ -1430,10 +1404,9 @@ struct XYSpinboxes
     return Process::DefaultControlLayouts::pad();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     SCORE_TODO;
     return nullptr; // TODO
@@ -1447,6 +1420,7 @@ struct XYSpinboxes
     if(slider.integral)
     {
       auto sl = new score::QGraphicsIntXYSpinboxChooser{false, nullptr};
+      initWidgetProperties(inlet, *sl);
       bindVec2Domain(slider, inlet, *sl);
       sl->setValue(ossia::convert<ossia::vec2f>(inlet.value()));
 
@@ -1454,7 +1428,8 @@ struct XYSpinboxes
           sl, &score::QGraphicsIntXYSpinboxChooser::sliderMoved, context,
           [=, &inlet, &ctx] {
         sl->moving = true;
-        ctx.dispatcher.submit<SetControlValue<Control_T>>(inlet, sl->value());
+        auto [x, y] = sl->value();
+        ctx.dispatcher.submit<SetControlValue<Control_T>>(inlet, ossia::make_vec(x, y));
       });
       QObject::connect(
           sl, &score::QGraphicsIntXYSpinboxChooser::sliderReleased, context,
@@ -1474,6 +1449,7 @@ struct XYSpinboxes
     else
     {
       auto sl = new score::QGraphicsXYSpinboxChooser{false, nullptr};
+      initWidgetProperties(inlet, *sl);
       bindVec2Domain(slider, inlet, *sl);
       sl->setValue(
           LinearNormalizer::to01(*sl, ossia::convert<ossia::vec2f>(inlet.value())));
@@ -1509,10 +1485,9 @@ struct XYZSpinboxes
     return Process::DefaultControlLayouts::pad();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     SCORE_TODO;
     return nullptr; // TODO
@@ -1524,6 +1499,7 @@ struct XYZSpinboxes
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsXYZSpinboxChooser{nullptr};
+    initWidgetProperties(inlet, *sl);
     bindVec3Domain(slider, inlet, *sl);
     sl->setValue(
         LinearNormalizer::to01(*sl, ossia::convert<ossia::vec3f>(inlet.value())));
@@ -1556,10 +1532,9 @@ struct MultiSlider
     return Process::DefaultControlLayouts::slider();
   }
 
-  template <typename T, typename Control_T>
+  template <typename T>
   static auto make_widget(
-      const T& slider, Control_T& inlet, const score::DocumentContext& ctx,
-      QWidget* parent, QObject* context)
+      T& inlet, const score::DocumentContext& ctx, QWidget* parent, QObject* context)
   {
     SCORE_TODO;
     return nullptr; // TODO
@@ -1571,6 +1546,7 @@ struct MultiSlider
       QGraphicsItem* parent, QObject* context)
   {
     auto sl = new score::QGraphicsMultiSlider{nullptr};
+    initWidgetProperties(inlet, *sl);
     sl->setValue(inlet.value());
     sl->setRange(inlet.domain());
 
