@@ -17,6 +17,7 @@
 #include <score/command/Dispatchers/CommandDispatcher.hpp>
 #include <score/document/DocumentContext.hpp>
 #include <score/graphics/GraphicWidgets.hpp>
+#include <score/graphics/GraphicsLayout.hpp>
 #include <score/graphics/TextItem.hpp>
 #include <score/model/Skin.hpp>
 #include <score/selection/SelectionDispatcher.hpp>
@@ -26,6 +27,7 @@
 #include <ossia-qt/invoke.hpp>
 
 #include <QCursor>
+#include <QGraphicsScene>
 #include <QGraphicsSceneMouseEvent>
 #include <QKeyEvent>
 #include <QPainter>
@@ -84,6 +86,14 @@ NodeItem::NodeItem(
     if(p != pos())
       setPos(p);
   });
+
+  auto on_sizeChanged = [this] {
+    updateSize();
+    updateZoomRatio();
+    updateTitlePos();
+  };
+  connect(&process, &Process::ProcessModel::inletsChanged, this, on_sizeChanged);
+  connect(&process, &Process::ProcessModel::outletsChanged, this, on_sizeChanged);
 }
 
 void NodeItem::createWithDecorations()
@@ -384,6 +394,16 @@ void NodeItem::createContentItem()
 
   if(resizeable)
   {
+    if(auto lay = dynamic_cast<score::GraphicsLayout*>(resizeable))
+      connect(lay, &score::ResizeableItem::childrenSizeChanged, this, [this, lay] {
+        lay->layout();
+        lay->fitChildrenRect();
+        m_contentSize = lay->rect().size();
+
+        updateSize();
+        updateZoomRatio();
+        updateTitlePos();
+      });
     connect(resizeable, &score::ResizeableItem::sizeChanged, this, [this](QSizeF sz) {
       double w = std::max(minimalContentWidth(), sz.width());
       double h = std::max(minimalContentHeight(), sz.height());
@@ -416,8 +436,11 @@ double NodeItem::minimalContentHeight() const noexcept
 void NodeItem::updateSize()
 {
   if(!m_label)
+  {
+    if(auto sc = this->scene())
+      sc->update();
     return;
-
+  }
   auto sz = m_fx ? m_fx->boundingRect().size()
                  : QSizeF{minimalContentWidth(), minimalContentHeight()};
 
@@ -447,6 +470,8 @@ void NodeItem::updateSize()
     }
     updateTitlePos();
     update();
+    if(auto sc = this->scene())
+      sc->update();
   }
 }
 
@@ -477,6 +502,8 @@ void NodeItem::setSize(QSizeF sz)
       m_uiButton->setPos({m_contentSize.width() + TopButtonX0, TopButtonY0});
     }
   }
+  if(auto sc = this->scene())
+    sc->update();
 }
 
 const Id<Process::ProcessModel>& NodeItem::id() const noexcept
