@@ -1,6 +1,7 @@
 #include "Port.hpp"
 
 #include <Process/Dataflow/Cable.hpp>
+#include <Process/Dataflow/ControlWidgets.hpp>
 #include <Process/Dataflow/PortFactory.hpp>
 #include <Process/Dataflow/PortItem.hpp>
 #include <Process/Dataflow/PortListWidget.hpp>
@@ -8,8 +9,6 @@
 #include <Process/Dataflow/PrettyPortName.hpp>
 #include <Process/ProcessContext.hpp>
 
-#include <Control/Widgets.hpp>
-
 #include <score/application/GUIApplicationContext.hpp>
 #include <score/graphics/GraphicsLayout.hpp>
 #include <score/graphics/RectItem.hpp>
@@ -24,6 +23,7 @@
 #include <ossia-qt/value_metatypes.hpp>
 
 #include <QDebug>
+#include <QIODevice>
 
 #include <wobjectimpl.h>
 W_OBJECT_IMPL(Process::Port)
@@ -270,6 +270,12 @@ void Inlet::mapExecution(
 {
 }
 
+ControlInlet::ControlInlet(const QString& name, Id<Process::Port> c, QObject* parent)
+    : Inlet{std::move(c), parent}
+{
+  setName(name);
+}
+
 ControlInlet::ControlInlet(Id<Process::Port> c, QObject* parent)
     : Inlet{std::move(c), parent}
 {
@@ -329,8 +335,19 @@ void ControlInlet::setValue(const ossia::value& value)
   }
 }
 
+void ControlInlet::setInit(const ossia::value& value)
+{
+  if(value != m_init)
+  {
+    m_init = value;
+    initChanged(value);
+  }
+}
+
 Outlet::~Outlet() { }
 
+void Outlet::setupExecution(ossia::outlet&) const noexcept { }
+
 Outlet::Outlet(Id<Process::Port> c, QObject* parent)
     : Port{std::move(c), QStringLiteral("Outlet"), parent}
 {
@@ -450,12 +467,23 @@ void AudioOutlet::mapExecution(
 
 QByteArray AudioOutlet::saveData() const noexcept
 {
-  return Port::saveData();
+  QByteArray arr;
+  {
+    QDataStream p{&arr, QIODevice::WriteOnly};
+    DataStreamInput ip{p};
+    ip << Port::saveData() << m_gain << m_pan << m_propagate;
+  }
+  return arr;
 }
 
 void AudioOutlet::loadData(const QByteArray& arr) noexcept
 {
-  Port::loadData(arr);
+  QByteArray pdata;
+  QDataStream p{arr};
+  DataStreamOutput op{p};
+  op >> pdata >> m_gain >> m_pan >> m_propagate;
+  Port::loadData(pdata);
+  propagateChanged(m_propagate);
 }
 
 bool AudioOutlet::propagate() const
@@ -558,6 +586,12 @@ MidiOutlet::MidiOutlet(JSONObject::Deserializer&& vis, QObject* parent)
   vis.writeTo(*this);
 }
 
+ControlOutlet::ControlOutlet(const QString& qname, Id<Process::Port> c, QObject* parent)
+    : Outlet{std::move(c), parent}
+{
+  setName(qname);
+}
+
 ControlOutlet::ControlOutlet(Id<Process::Port> c, QObject* parent)
     : Outlet{std::move(c), parent}
 {
@@ -673,14 +707,14 @@ void PortFactory::setupInletInspector(
     const Inlet& port, const score::DocumentContext& ctx, QWidget* parent,
     Inspector::Layout& lay, QObject* context)
 {
-  PortWidgetSetup::setupInLayout(port, ctx, lay, parent);
+  PortWidgetSetup::setupAlone(port, ctx, lay, parent);
 }
 
 void PortFactory::setupOutletInspector(
     const Outlet& port, const score::DocumentContext& ctx, QWidget* parent,
     Inspector::Layout& lay, QObject* context)
 {
-  PortWidgetSetup::setupInLayout(port, ctx, lay, parent);
+  PortWidgetSetup::setupAlone(port, ctx, lay, parent);
 }
 
 PortItemLayout PortFactory::defaultLayout() const noexcept
@@ -700,10 +734,11 @@ QGraphicsItem* PortFactory::makeControlItem(
     auto max = dom.convert_max<float>();
     struct
     {
-      float min, max;
+      float min, max, init;
       float getMin() const { return min; }
       float getMax() const { return max; }
-    } info{min, max};
+      float getInit() const { return init; }
+    } info{min, max, ossia::convert<float>(port.init())};
     return WidgetFactory::FloatSlider::make_item(info, port, ctx, nullptr, context);
   }
   else
@@ -712,6 +747,7 @@ QGraphicsItem* PortFactory::makeControlItem(
     {
       static float getMin() { return 0.; }
       static float getMax() { return 1.; }
+      static float getInit() { return 0.; }
     };
     return WidgetFactory::FloatSlider::make_item(
         SliderInfo{}, port, ctx, nullptr, context);
@@ -732,6 +768,7 @@ QGraphicsItem* PortFactory::makeControlItem(
       float min, max;
       float getMin() const { return min; }
       float getMax() const { return max; }
+      float getInit() const { return min; }
     } info{min, max};
     return WidgetFactory::FloatSlider::make_item(info, port, ctx, parent, context);
   }
@@ -741,16 +778,18 @@ QGraphicsItem* PortFactory::makeControlItem(
     {
       static float getMin() { return 0.; }
       static float getMax() { return 1.; }
+      static float getInit() { return 0.; }
     };
     return WidgetFactory::FloatSlider::make_item(
         SliderInfo{}, port, ctx, parent, context);
   }
 }
 
-auto makeFullItemImpl(
+static auto makeFullItemImpl(
     const Process::Port& portModel, const Process::PortItemLayout& layout,
     QGraphicsItem& port, QGraphicsItem& control, score::EmptyRectItem& item)
 {
+  score::SimpleTextItem* lab{};
   using namespace score;
 
   // Port
@@ -766,7 +805,7 @@ auto makeFullItemImpl(
   // Text
   if(layout.labelVisible)
   {
-    auto lab = Dataflow::makePortLabel(portModel, &item);
+    lab = Dataflow::makePortLabel(portModel, &item);
     lab->setPos(layout.label);
     if(layout.labelAlignment == Qt::AlignCenter)
     {
@@ -783,11 +822,14 @@ auto makeFullItemImpl(
     }
   }
   item.fitChildrenRect();
+  return lab;
 }
-QGraphicsItem* PortFactory::makeFullItem(
+
+Process::ControlLayout PortFactory::makeFullItem(
     ControlInlet& portModel, const Process::Context& ctx, QGraphicsItem* parent,
     QObject* context)
 {
+  Process::ControlLayout ret;
   using namespace score;
 #if defined(SCORE_DEBUG_CONTROL_RECTS)
   auto item = new score::GraphicsLayout{parent};
@@ -795,29 +837,32 @@ QGraphicsItem* PortFactory::makeFullItem(
 #else
   auto item = new score::EmptyRectItem{parent};
 #endif
+  ret.container = item;
 
   const auto& layout = defaultLayout();
-  auto port = makePortItem(portModel, ctx, item, context);
-  SCORE_SOFT_ASSERT(port);
+  ret.port_item = makePortItem(portModel, ctx, ret.container, context);
+  SCORE_SOFT_ASSERT(ret.port_item);
 
-  if(port)
+  if(ret.port_item)
   {
-    auto widg = makeControlItem(portModel, ctx, item, context);
-    SCORE_SOFT_ASSERT(widg);
+    ret.control = makeControlItem(portModel, ctx, ret.container, context);
+    SCORE_SOFT_ASSERT(ret.control);
 
-    if(widg)
+    if(ret.control)
     {
-      makeFullItemImpl(portModel, layout, *port, *widg, *item);
+      ret.label
+          = makeFullItemImpl(portModel, layout, *ret.port_item, *ret.control, *item);
     }
   }
 
-  return item;
+  return ret;
 }
 
-QGraphicsItem* PortFactory::makeFullItem(
+Process::ControlLayout PortFactory::makeFullItem(
     ControlOutlet& portModel, const Process::Context& ctx, QGraphicsItem* parent,
     QObject* context)
 {
+  Process::ControlLayout ret;
   using namespace score;
 #if defined(SCORE_DEBUG_CONTROL_RECTS)
   auto item = new score::GraphicsLayout{parent};
@@ -825,14 +870,14 @@ QGraphicsItem* PortFactory::makeFullItem(
 #else
   auto item = new score::EmptyRectItem{parent};
 #endif
+  ret.container = item;
 
   const auto& layout = defaultLayout();
-  auto port = makePortItem(portModel, ctx, item, context);
-  auto widg = makeControlItem(portModel, ctx, item, context);
+  ret.port_item = makePortItem(portModel, ctx, item, context);
+  ret.control = makeControlItem(portModel, ctx, item, context);
+  ret.label = makeFullItemImpl(portModel, layout, *ret.port_item, *ret.control, *item);
 
-  makeFullItemImpl(portModel, layout, *port, *widg, *item);
-
-  return item;
+  return ret;
 }
 
 Port* PortFactoryList::loadMissing(const VisitorVariant& vis, QObject* parent) const
@@ -1163,12 +1208,14 @@ SCORE_LIB_PROCESS_EXPORT void DataStreamReader::read(const Process::ControlInlet
 {
   // read((Process::Inlet&)p);
   readFrom(p.m_value);
+  readFrom(p.m_init);
   readFrom(p.m_domain);
 }
 template <>
 SCORE_LIB_PROCESS_EXPORT void DataStreamWriter::write(Process::ControlInlet& p)
 {
   writeTo(p.m_value);
+  writeTo(p.m_init);
   writeTo(p.m_domain);
 }
 
@@ -1177,12 +1224,16 @@ SCORE_LIB_PROCESS_EXPORT void JSONReader::read(const Process::ControlInlet& p)
 {
   // read((Process::Inlet&)p);
   obj[strings.Value] = p.m_value;
+  obj[strings.Init] = p.m_init;
   obj[strings.Domain] = p.m_domain;
 }
+
 template <>
 SCORE_LIB_PROCESS_EXPORT void JSONWriter::write(Process::ControlInlet& p)
 {
   p.m_value <<= obj[strings.Value];
+  if(auto v = obj.tryGet(strings.Init))
+    p.m_init <<= *v;
   p.m_domain <<= obj[strings.Domain];
 }
 
