@@ -15,6 +15,7 @@
 #include <boost/container/vector.hpp>
 
 #include <avnd/binding/ossia/qt.hpp>
+#include <avnd/binding/ossia/uuid.hpp>
 #include <avnd/common/concepts_polyfill.hpp>
 #include <avnd/common/struct_reflection.hpp>
 #include <avnd/concepts/audio_port.hpp>
@@ -23,6 +24,7 @@
 #include <avnd/concepts/gfx.hpp>
 #include <avnd/concepts/midi_port.hpp>
 #include <avnd/concepts/parameter.hpp>
+#include <avnd/introspection/input.hpp>
 #include <avnd/wrappers/metadatas.hpp>
 #include <avnd/wrappers/widgets.hpp>
 
@@ -31,11 +33,6 @@
 #include <type_traits>
 
 #define make_uuid(text) score::uuids::string_generator::compute((text))
-#if defined(_MSC_VER)
-#define uuid_constexpr inline
-#else
-#define uuid_constexpr constexpr
-#endif
 
 namespace oscr
 {
@@ -50,39 +47,6 @@ struct is_custom_serialized<oscr::CustomFloatControl<Node, FieldIndex>> : std::t
 
 namespace oscr
 {
-template <typename N>
-consteval score::uuid_t uuid_from_string()
-{
-  if constexpr(requires {
-                 {
-                   N::uuid()
-                 } -> std::convertible_to<score::uuid_t>;
-               })
-  {
-    return N::uuid();
-  }
-  else
-  {
-    constexpr const char* str = N::uuid();
-    return score::uuids::string_generator::compute(str, str + 37);
-  }
-}
-
-template <typename Node>
-score::uuids::uuid make_field_uuid(uint64_t is_input, uint64_t index)
-{
-  score::uuid_t node_uuid = uuid_from_string<Node>();
-  uint64_t dat[2];
-
-  memcpy(dat, node_uuid.data, 16);
-
-  dat[0] ^= is_input;
-  dat[1] ^= index;
-
-  memcpy(node_uuid.data, dat, 16);
-
-  return node_uuid;
-}
 
 struct CustomFloatControlBase : public Process::ControlInlet
 {
@@ -95,6 +59,7 @@ struct CustomFloatControlBase : public Process::ControlInlet
   {
     hidden = true;
     setValue(init);
+    setInit(init);
     setDomain(ossia::make_domain(min, max));
     setName(name);
   }
@@ -158,7 +123,8 @@ namespace oscr
 {
 
 template <typename Node, typename T, std::size_t N>
-auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* parent)
+static inline auto
+make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* parent)
 {
   using value_type = as_type(T::value);
   constexpr auto name = avnd::get_name<T>();
@@ -182,7 +148,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::toggle)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(requires { c.values(); })
     {
       return new Process::ChooserToggle{
@@ -195,7 +161,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::slider)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(std::is_integral_v<value_type>)
     {
       return new Process::IntSlider{c.min, c.max, c.init, qname, id, parent};
@@ -213,14 +179,25 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
       }
     }
   }
+  else if constexpr(widg.widget == avnd::widget_type::log_slider)
+  {
+    static constexpr auto c = avnd::get_range<T>();
+    return new Process::LogFloatSlider{c.min, c.max, c.init, qname, id, parent};
+  }
+  else if constexpr(widg.widget == avnd::widget_type::log_knob)
+  {
+    // FIXME
+    static constexpr auto c = avnd::get_range<T>();
+    return new Process::LogFloatSlider{c.min, c.max, c.init, qname, id, parent};
+  }
   else if constexpr(widg.widget == avnd::widget_type::time_chooser)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     return new Process::TimeChooser{c.min, c.max, c.init, qname, id, parent};
   }
   else if constexpr(widg.widget == avnd::widget_type::range_slider)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(std::is_integral_v<value_type>)
     {
       auto [start, end] = c.init;
@@ -236,7 +213,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::range_spinbox)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(std::is_integral_v<value_type>)
     {
       auto [start, end] = c.init;
@@ -250,9 +227,16 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
                                             qname, id,    parent};
     }
   }
-  else if constexpr(widg.widget == avnd::widget_type::spinbox)
+  else if constexpr(widg.widget == avnd::widget_type::multi_slider)
   {
     constexpr auto c = avnd::get_range<T>();
+    auto [start, end] = c.init;
+    std::vector<ossia::value> init;
+    return new Process::MultiSlider{init, qname, id, parent};
+  }
+  else if constexpr(widg.widget == avnd::widget_type::spinbox)
+  {
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(std::is_integral_v<value_type>)
     {
       return new Process::IntSpinBox{c.min, c.max, c.init, qname, id, parent};
@@ -264,7 +248,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::knob)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(std::is_integral_v<value_type>)
     {
       // FIXME do a IntKnob
@@ -287,7 +271,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   {
     if constexpr(avnd::has_range<T>)
     {
-      constexpr auto c = avnd::get_range<T>();
+      static constexpr auto c = avnd::get_range<T>();
       if constexpr(avnd::program_parameter<T>)
       {
         auto p = new Process::ProgramEdit{c.init.data(), qname, id, parent};
@@ -311,18 +295,20 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::combobox)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     return new Process::ComboBox{to_combobox_range(c.values), c.init, qname, id, parent};
   }
   else if constexpr(widg.widget == avnd::widget_type::choices)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     auto enums = avnd::to_enum_range(c.values);
-    auto init = enums[c.init];
+    static_assert(
+        std::is_integral_v<decltype(c.init)> || std::is_enum_v<decltype(c.init)>);
+    auto init = enums[static_cast<int>(c.init)];
     std::vector<QString> pixmaps;
-    if constexpr(requires { c.pixmaps; })
+    if constexpr(avnd::has_pixmaps<T>)
     {
-      for(auto& pix : c.pixmaps)
+      for(std::string_view pix : avnd::get_pixmaps<T>())
       {
         pixmaps.push_back(QString::fromLatin1(pix.data(), pix.size()));
       }
@@ -332,7 +318,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::xy)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(requires {
                    c.min == 0.f;
                    c.max == 0.f;
@@ -352,7 +338,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::xyz)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(requires {
                    c.min == 0.f;
                    c.max == 0.f;
@@ -379,7 +365,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   else if constexpr(widg.widget == avnd::widget_type::xy_spinbox)
   {
     using data_type = std::decay_t<decltype(value_type{}.x)>;
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(requires {
                    c.min == 0.f;
                    c.max == 0.f;
@@ -407,7 +393,7 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::xyz_spinbox)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     if constexpr(requires {
                    c.min == 0.f;
                    c.max == 0.f;
@@ -433,10 +419,20 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
   }
   else if constexpr(widg.widget == avnd::widget_type::color)
   {
-    constexpr auto c = avnd::get_range<T>();
-    constexpr auto i = c.init;
+    static constexpr auto c = avnd::get_range<T>();
+    static constexpr auto i = c.init;
     return new Process::HSVSlider{{i.r, i.g, i.b, i.a}, qname, id, parent};
   }
+  else if constexpr(widg.widget == avnd::widget_type::control)
+  {
+    return new Process::ControlInlet{qname, id, parent};
+  }
+  else if constexpr(widg.widget == avnd::widget_type::no_control)
+  {
+    auto pv = new Process::ValueInlet{id, parent};
+    pv->setName(qname);
+    return pv;
+  }
   else
   {
     static_assert(T::is_not_a_valid_control);
@@ -444,7 +440,8 @@ auto make_control_in(avnd::field_index<N>, Id<Process::Port>&& id, QObject* pare
 }
 
 template <typename T, std::size_t N>
-auto make_control_out(avnd::field_index<N>, Id<Process::Port>&& id, QObject* parent)
+static inline auto
+make_control_out(avnd::field_index<N>, Id<Process::Port>&& id, QObject* parent)
 {
   constexpr auto name = avnd::get_name<T>();
   constexpr auto widg = avnd::get_widget<T>();
@@ -454,22 +451,32 @@ auto make_control_out(avnd::field_index<N>, Id<Process::Port>&& id, QObject* par
 
   if constexpr(widg.widget == avnd::widget_type::bargraph)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     return new Process::Bargraph{c.min, c.max, c.init, qname, id, parent};
   }
+  else if constexpr(widg.widget == avnd::widget_type::control)
+  {
+    return new Process::ControlOutlet{qname, id, parent};
+  }
+  else if constexpr(widg.widget == avnd::widget_type::no_control)
+  {
+    auto pv = new Process::ValueOutlet{id, parent};
+    pv->setName(qname);
+    return pv;
+  }
   else if constexpr(avnd::fp_ish<decltype(T::value)>)
   {
-    constexpr auto c = avnd::get_range<T>();
+    static constexpr auto c = avnd::get_range<T>();
     return new Process::Bargraph{c.min, c.max, c.init, qname, id, parent};
   }
   else
   {
-    static_assert(T::is_not_a_valid_control);
+    return new Process::ControlOutlet{qname, id, parent};
   }
 }
 
 template <typename T>
-constexpr auto make_control_out(const T& t)
+static inline constexpr auto make_control_out(const T& t)
 {
   return make_control_out<T>();
 }
