@@ -15,6 +15,7 @@ struct GfxRenderer<Node_T> final : score::gfx::OutputNodeRenderer
   using texture_inputs = avnd::texture_input_introspection<Node_T>;
   const GfxNode<Node_T>& parent;
   Node_T state;
+  score::gfx::Message m_last_message{};
   ossia::small_flat_map<const score::gfx::Port*, score::gfx::TextureRenderTarget, 2>
       m_rts;
 
@@ -42,7 +43,8 @@ struct GfxRenderer<Node_T> final : score::gfx::OutputNodeRenderer
       score::gfx::RenderList& renderer, int k, const Tex& texture_spec, QSize size)
   {
     auto port = parent.input[k];
-    constexpr auto flags = QRhiTexture::RenderTarget | QRhiTexture::UsedAsTransferSource;
+    static constexpr auto flags
+        = QRhiTexture::RenderTarget | QRhiTexture::UsedAsTransferSource;
     auto texture = renderer.state.rhi->newTexture(
         gpp::qrhi::textureFormat<Tex>(), size, 1, flags);
     SCORE_ASSERT(texture->create());
@@ -75,10 +77,11 @@ struct GfxRenderer<Node_T> final : score::gfx::OutputNodeRenderer
 
   void init(score::gfx::RenderList& renderer, QRhiResourceUpdateBatch& res) override
   {
-    if constexpr(requires { state.init(); })
+    if constexpr(requires { state.prepare(); })
     {
-      parent.processControlIn(state, this->parent.last_message);
-      state.init();
+      parent.processControlIn(
+          *this, state, m_last_message, this->parent.last_message, this->parent.m_ctx);
+      state.prepare();
     }
 
     // Init input render targets
@@ -192,7 +195,8 @@ struct GfxRenderer<Node_T> final : score::gfx::OutputNodeRenderer
           });
     }
 
-    parent.processControlIn(state, this->parent.last_message);
+    parent.processControlIn(
+        *this, state, m_last_message, this->parent.last_message, this->parent.m_ctx);
 
     // Run the processor
     state();
@@ -203,16 +207,18 @@ struct GfxRenderer<Node_T> final : score::gfx::OutputNodeRenderer
 };
 
 template <typename Node_T>
-  requires(
-      avnd::texture_input_introspection<Node_T>::size > 0
-      && avnd::texture_output_introspection<Node_T>::size == 0)
-struct GfxNode<Node_T> final : CustomGpuOutputNodeBase
+  requires(avnd::texture_input_introspection<Node_T>::size > 0
+           && avnd::texture_output_introspection<Node_T>::size == 0)
+struct GfxNode<Node_T> final
+    : CustomGpuOutputNodeBase
+    , GpuNodeElements<Node_T>
 {
   oscr::ProcessModel<Node_T>& processModel;
   GfxNode(
       oscr::ProcessModel<Node_T>& element,
-      std::weak_ptr<Execution::ExecutionCommandQueue> q, Gfx::exec_controls ctls, int id)
-      : CustomGpuOutputNodeBase{std::move(q), std::move(ctls)}
+      std::weak_ptr<Execution::ExecutionCommandQueue> q, Gfx::exec_controls ctls, int id,
+      const score::DocumentContext& ctx)
+      : CustomGpuOutputNodeBase{std::move(q), std::move(ctls), ctx}
       , processModel{element}
   {
     this->instance = id;
