@@ -16,6 +16,7 @@ struct GfxRenderer<Node_T> final : score::gfx::GenericNodeRenderer
   using texture_outputs = avnd::texture_output_introspection<Node_T>;
   const GfxNode<Node_T>& parent;
   Node_T state;
+  score::gfx::Message m_last_message{};
   ossia::small_flat_map<const score::gfx::Port*, score::gfx::TextureRenderTarget, 2>
       m_rts;
 
@@ -43,7 +44,8 @@ struct GfxRenderer<Node_T> final : score::gfx::GenericNodeRenderer
       score::gfx::RenderList& renderer, int k, const Tex& texture_spec, QSize size)
   {
     auto port = parent.input[k];
-    constexpr auto flags = QRhiTexture::RenderTarget | QRhiTexture::UsedAsTransferSource;
+    static constexpr auto flags
+        = QRhiTexture::RenderTarget | QRhiTexture::UsedAsTransferSource;
     auto texture = renderer.state.rhi->newTexture(
         gpp::qrhi::textureFormat<Tex>(), size, 1, flags);
     SCORE_ASSERT(texture->create());
@@ -153,9 +155,11 @@ struct GfxRenderer<Node_T> final : score::gfx::GenericNodeRenderer
 
   void init(score::gfx::RenderList& renderer, QRhiResourceUpdateBatch& res) override
   {
-    if constexpr(requires { state.init(); })
+    if constexpr(requires { state.prepare(); })
     {
-      state.init();
+      parent.processControlIn(
+          *this, state, m_last_message, this->parent.last_message, this->parent.m_ctx);
+      state.prepare();
     }
 
     const auto& mesh = renderer.defaultTriangle();
@@ -171,6 +175,14 @@ struct GfxRenderer<Node_T> final : score::gfx::GenericNodeRenderer
       avnd::cpu_texture_input_introspection<Node_T>::for_all(
           avnd::get_inputs<Node_T>(state), [&]<typename F>(F& t) {
             auto sz = renderer.state.renderSize;
+            if constexpr(requires {
+                           t.request_width;
+                           t.request_height;
+                         })
+            {
+              sz.rwidth() = t.request_width;
+              sz.rheight() = t.request_height;
+            }
             createInput(renderer, k, t.texture, sz);
             t.texture.width = sz.width();
             t.texture.height = sz.height();
@@ -258,7 +270,8 @@ struct GfxRenderer<Node_T> final : score::gfx::GenericNodeRenderer
           });
     }
 
-    parent.processControlIn(state, this->parent.last_message);
+    parent.processControlIn(
+        *this, state, m_last_message, this->parent.last_message, this->parent.m_ctx);
 
     // Run the processor
     state();
@@ -289,12 +302,15 @@ struct GfxNode<Node_T> final
     , GpuWorker
     , GpuControlIns
     , GpuControlOuts
+    , GpuNodeElements<Node_T>
 {
   oscr::ProcessModel<Node_T>& processModel;
   GfxNode(
       oscr::ProcessModel<Node_T>& element,
-      std::weak_ptr<Execution::ExecutionCommandQueue> q, Gfx::exec_controls ctls, int id)
-      : GpuControlOuts{std::move(q), std::move(ctls)}
+      std::weak_ptr<Execution::ExecutionCommandQueue> q, Gfx::exec_controls ctls, int id,
+      const score::DocumentContext& ctx)
+      : CustomGfxNodeBase{ctx}
+      , GpuControlOuts{std::move(q), std::move(ctls)}
       , processModel{element}
   {
     this->instance = id;
