@@ -12,6 +12,7 @@
 #include <score/model/Skin.hpp>
 #include <score/plugins/InterfaceList.hpp>
 #include <score/plugins/StringFactoryKey.hpp>
+#include <score/widgets/MarginLess.hpp>
 #include <score/widgets/SignalUtils.hpp>
 #include <score/widgets/TextLabel.hpp>
 
@@ -20,6 +21,7 @@
 #include <ossia/detail/algorithms.hpp>
 
 #include <QComboBox>
+#include <QDesktopServices>
 #include <QDialogButtonBox>
 #include <QFormLayout>
 #include <QHeaderView>
@@ -48,6 +50,7 @@ DeviceEditDialog::DeviceEditDialog(
     , m_protocolWidget{nullptr}
     , m_index{-1}
 {
+
   const auto& skin = score::Skin::instance();
   const QColor textHeaderColor = QColor("#D5D5D5");
   auto setHeaderTextFormat = [&](QLabel* label) {
@@ -58,65 +61,104 @@ DeviceEditDialog::DeviceEditDialog(
   };
 
   setWindowTitle(tr("Add device"));
-  auto gridLayout = new QGridLayout{};
-  setLayout(gridLayout);
+  auto base_layout = new QHBoxLayout{this};
+  setLayout(base_layout);
   setModal(true);
 
+  m_splitter = new QSplitter{this};
+  base_layout->addWidget(m_splitter);
+
+  auto column1 = new QWidget;
+  auto column1_layout = new score::MarginLess<QVBoxLayout>{column1};
+  m_protocolsLabel = new QLabel{tr("Protocols"), this};
+  setHeaderTextFormat(m_protocolsLabel);
+  column1_layout->addWidget(m_protocolsLabel);
+  m_protocolsLabel->setAlignment(Qt::AlignTop);
+  m_protocolsLabel->setAlignment(Qt::AlignHCenter);
+  m_protocols = new QTreeWidget{this};
+  m_protocols->header()->hide();
+  m_protocols->setSelectionMode(QAbstractItemView::SingleSelection);
+  column1_layout->addWidget(m_protocols);
+  column1->setLayout(column1_layout);
+  column1->setFixedWidth(200);
+  base_layout->addWidget(column1);
+
+  if(m_mode == Mode::Editing)
+  {
+    column1->setVisible(false);
+  }
+
+  base_layout->addWidget(m_splitter);
+
+  // Column 2: Devices
+  auto column2 = new QWidget;
+  auto column2_layout = new score::MarginLess<QVBoxLayout>{column2};
+  m_devicesLabel = new QLabel{tr("Devices"), this};
+  setHeaderTextFormat(m_devicesLabel);
+  column2_layout->addWidget(m_devicesLabel);
+  m_devicesLabel->setAlignment(Qt::AlignTop);
+  m_devicesLabel->setAlignment(Qt::AlignHCenter);
+  m_devices = new QTreeWidget{this};
+  m_devices->header()->hide();
+  m_devices->setSelectionMode(QAbstractItemView::SingleSelection);
+  column2_layout->addWidget(m_devices);
+  column2->setLayout(column2_layout);
+  m_splitter->addWidget(column2);
+
+  // Column 3: Settings
+  auto column3 = new QWidget;
+  auto column3_layout = new score::MarginLess<QVBoxLayout>{column3};
+  m_protocolNameLabel = new QLabel{tr("Settings"), this};
+  setHeaderTextFormat(m_protocolNameLabel);
+  column3_layout->addWidget(m_protocolNameLabel);
+  m_protocolNameLabel->setAlignment(Qt::AlignTop);
+  m_protocolNameLabel->setAlignment(Qt::AlignHCenter);
+  m_main = new QWidget{this};
+  m_settingsFormLayout = new QFormLayout;
+  m_settingsFormLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
+  m_main->setLayout(m_settingsFormLayout);
+  column3_layout->addWidget(m_main);
+
   m_invalidLabel = new QLabel{
       tr("Cannot add device.\n Try changing the name to make it unique, \nor "
          "check that the ports aren't already used")};
   m_invalidLabel->setAlignment(Qt::AlignRight);
   m_invalidLabel->setTextFormat(Qt::PlainText);
+
   m_buttonBox = new QDialogButtonBox(Qt::Horizontal, this);
+  m_helpButton = m_buttonBox->addButton(tr("Help"), QDialogButtonBox::HelpRole);
   m_okButton = m_buttonBox->addButton(tr("Add"), QDialogButtonBox::AcceptRole);
   m_buttonBox->addButton(QDialogButtonBox::Cancel);
+  column3_layout->addStretch(1);
+  column3_layout->addWidget(m_invalidLabel);
+  column3_layout->addWidget(m_buttonBox);
+  column3->setLayout(column3_layout);
+  m_splitter->addWidget(column3);
+
+  m_devices->setMinimumWidth(40);
+  m_main->setMinimumWidth(100);
+
+  m_splitter->setCollapsible(0, false);
+  m_splitter->setCollapsible(1, false);
+
+  m_splitter->setStretchFactor(0, 1);
+  m_splitter->setStretchFactor(1, 2);
 
   connect(m_buttonBox, &QDialogButtonBox::accepted, this, &QDialog::accept);
   connect(m_buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
+  connect(m_buttonBox, &QDialogButtonBox::helpRequested, [this] {
+    if(!this->m_protocols)
+      return;
+    auto selected_item = m_protocols->selectedItems().first();
+    auto key
+        = selected_item->data(0, Qt::UserRole).value<UuidKey<Device::ProtocolFactory>>();
+    if(key == UuidKey<Device::ProtocolFactory>{})
+      return;
 
-  m_protocols = new QTreeWidget{this};
-  m_protocols->header()->hide();
-  m_protocols->setSelectionMode(QAbstractItemView::SingleSelection);
-  if(m_mode == Mode::Editing)
-  {
-    m_protocols->setVisible(false);
-  }
-  else
-  {
-    m_protocols->setFixedWidth(150);
-
-    {
-      m_protocolsLabel = new QLabel{tr("Protocols"), this};
-      setHeaderTextFormat(m_protocolsLabel);
-      gridLayout->addWidget(m_protocolsLabel, 0, 0, Qt::AlignHCenter);
-    }
-    gridLayout->addWidget(m_protocols, 1, 0);
-  }
-
-  m_devices = new QTreeWidget{this};
-  m_devices->header()->hide();
-  m_devices->setSelectionMode(QAbstractItemView::SingleSelection);
-  // m_devices->setFixedWidth(140);
-  gridLayout->addWidget(m_devices, 1, 1);
-  {
-    m_devicesLabel = new QLabel{tr("Devices"), this};
-    setHeaderTextFormat(m_devicesLabel);
-    gridLayout->addWidget(m_devicesLabel, 0, 1, Qt::AlignHCenter);
-  }
-
-  {
-    m_protocolNameLabel = new QLabel{tr("Settings"), this};
-    setHeaderTextFormat(m_protocolNameLabel);
-    gridLayout->addWidget(
-        m_protocolNameLabel, 0, 2, -1, -1, Qt::AlignLeft | Qt::AlignTop);
-  }
-  m_main = new QWidget{this};
-  gridLayout->addWidget(m_main, 1, 2, -1, -1);
-  m_layout = new QFormLayout;
-  m_layout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
-  m_main->setLayout(m_layout);
-  m_layout->addRow(m_invalidLabel);
-  m_layout->addRow(m_buttonBox);
+    if(auto* proto = m_protocolList.get(key))
+      if(auto manual = proto->manual(); !manual.isEmpty())
+        QDesktopServices::openUrl(manual);
+  });
 
   initAvailableProtocols();
 
@@ -126,13 +168,11 @@ DeviceEditDialog::DeviceEditDialog(
 
   if(m_protocols->topLevelItemCount() > 0)
   {
-    //  m_protocols->setCurrentItem();
-    //  m_index = m_protocols->currentRow();
     selectedProtocolChanged();
   }
 
-  setMinimumWidth(700);
-  setMinimumHeight(400);
+  setMinimumWidth(850);
+  setMinimumHeight(550);
 
   setAcceptEnabled(false);
 }
@@ -201,7 +241,6 @@ void DeviceEditDialog::selectedDeviceChanged()
 {
   if(!m_devices->isVisible())
     return;
-
   if(m_devices->selectedItems().isEmpty())
     return;
 
@@ -211,6 +250,7 @@ void DeviceEditDialog::selectedDeviceChanged()
 
   auto name = item->text(0);
   auto data = item->data(0, Qt::UserRole).value<Device::DeviceSettings>();
+
   if(m_protocolWidget)
     m_protocolWidget->setSettings(data);
 
@@ -226,8 +266,6 @@ void DeviceEditDialog::selectedProtocolChanged()
   // Recreate
   if(m_protocols->selectedItems().isEmpty())
   {
-    //m_devices->setVisible(false);
-    //m_devicesLabel->setVisible(false);
     return;
   }
   auto selected_item = m_protocols->selectedItems().first();
@@ -246,7 +284,6 @@ void DeviceEditDialog::selectedProtocolChanged()
   if(m_protocolWidget)
   {
     SCORE_ASSERT(m_index < m_previousSettings.count());
-
     m_previousSettings[m_index] = m_protocolWidget->getSettings();
     delete m_protocolWidget;
     m_protocolWidget = nullptr;
@@ -261,6 +298,11 @@ void DeviceEditDialog::selectedProtocolChanged()
     m_devicesLabel->setVisible(true);
     m_devices->setRootIsDecorated(false);
     m_devices->setExpandsOnDoubleClick(false);
+    if(m_splitter->count() > 0)
+    {
+      m_splitter->widget(0)->show();
+      m_splitter->widget(0)->setMinimumWidth(200);
+    }
 
     for(auto& [name, e] : m_enumerators)
     {
@@ -304,6 +346,7 @@ void DeviceEditDialog::selectedProtocolChanged()
   {
     m_devices->setVisible(false);
     m_devicesLabel->setVisible(false);
+    m_splitter->widget(0)->hide();
   }
   m_protocolNameLabel->setText(tr("Settings (%1)").arg(protocol->prettyName()));
   m_protocolWidget = protocol->makeSettingsWidget();
@@ -315,7 +358,7 @@ void DeviceEditDialog::selectedProtocolChanged()
         m_protocolWidget, &Device::ProtocolSettingsWidget::changed, this,
         &DeviceEditDialog::updateValidity);
 
-    m_layout->insertRow(0, m_protocolWidget);
+    m_settingsFormLayout->insertRow(0, m_protocolWidget);
 
     QSizePolicy pol{QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding};
     pol.setVerticalStretch(1);
