@@ -112,6 +112,11 @@ QString CameraProtocolFactory::prettyName() const noexcept
   return QObject::tr("Camera input");
 }
 
+QUrl CameraProtocolFactory::manual() const noexcept
+{
+  return QUrl("https://ossia.io/score-docs/devices/camera-device.html");
+}
+
 QString CameraProtocolFactory::category() const noexcept
 {
   return StandardCategories::video;
@@ -180,7 +185,7 @@ void CameraProtocolFactory::serializeProtocolSpecificSettings(
 bool CameraProtocolFactory::checkCompatibility(
     const Device::DeviceSettings& a, const Device::DeviceSettings& b) const noexcept
 {
-  return a.name != b.name;
+  return true;
 }
 
 class CameraSettingsWidget final : public Device::ProtocolSettingsWidget
