@@ -15,6 +15,7 @@
 #include <QDialogButtonBox>
 #include <QFormLayout>
 #include <QObject>
+#include <QUrl>
 
 namespace Protocols
 {
@@ -24,6 +25,11 @@ QString ArtnetProtocolFactory::prettyName() const noexcept
   return QObject::tr("Artnet");
 }
 
+QUrl ArtnetProtocolFactory::manual() const noexcept
+{
+  return QUrl("https://ossia.io/score-docs/devices/artnet-device.html");
+}
+
 QString ArtnetProtocolFactory::category() const noexcept
 {
   return StandardCategories::lights;
@@ -70,10 +76,6 @@ void ArtnetProtocolFactory::serializeProtocolSpecificSettings(
 bool ArtnetProtocolFactory::checkCompatibility(
     const Device::DeviceSettings& a, const Device::DeviceSettings& b) const noexcept
 {
-  if(a.name == b.name)
-    return false;
-  if(a.protocol != b.protocol)
-    return true;
   auto lhs = a.deviceSpecificSettings.value<ArtnetSpecificSettings>();
   auto rhs = b.deviceSpecificSettings.value<ArtnetSpecificSettings>();
   if(lhs.transport != rhs.transport)
