@@ -14,9 +14,13 @@
 
 #include <score/application/GUIApplicationContext.hpp>
 
-#include <QObject>
-#include <libremidi/libremidi.hpp>
+#include <ossia-qt/invoke.hpp>
 
+#include <QObject>
+#include <QUrl>
+
+#include <libremidi/backends.hpp>
+#include <libremidi/libremidi.hpp>
 namespace Device
 {
 class DeviceInterface;
@@ -60,34 +64,47 @@ to_settings(libremidi::API api, const libremidi::output_port& p)
 
   return set;
 }
+
+static auto getCurrentAPI()
+{
+  auto api
+      = score::AppContext().settings<Protocols::Settings::Model>().getMidiApiAsEnum();
+  if(api == libremidi::API::UNSPECIFIED)
+    api = libremidi::midi1::default_api();
+  return api;
+}
+
 template <ossia::net::midi::midi_info::Type Type>
 class MidiEnumerator : public Device::DeviceEnumerator
 {
-  libremidi::API m_api = [] {
-    auto api
-        = score::AppContext().settings<Protocols::Settings::Model>().getMidiApiAsEnum();
-    if(api == libremidi::API::UNSPECIFIED)
-      api = libremidi::midi1::default_api();
-    return api;
+  libremidi::API m_api = getCurrentAPI();
+  std::any m_observer_conf = [this] {
+    auto api_conf = libremidi::observer_configuration_for(m_api);
+
+    libremidi::midi_any::for_observer_configuration([&](auto& conf) {
+      if constexpr(requires { conf.client_name; })
+        conf.client_name = "ossia score";
+    }, api_conf);
+
+    return api_conf;
   }();
 
-  libremidi::observer_configuration make_callbacks()
+  libremidi::observer_configuration make_callbacks(libremidi::observer_configuration& cb)
   {
-    libremidi::observer_configuration cb;
-    cb.notify_in_constructor = false;
+    cb.notify_in_constructor = true;
 
     if constexpr(Type == ossia::net::midi::midi_info::Type::Input)
     {
       cb.input_added = [this](const libremidi::input_port& p) {
-        auto s = to_settings(m_api, p);
-        deviceAdded(s.name, s);
+        ossia::qt::run_async(
+            this, [this, s = to_settings(m_api, p)] { deviceAdded(s.name, s); });
       };
     }
     else
     {
       cb.output_added = [this](const libremidi::output_port& p) {
-        auto s = to_settings(m_api, p);
-        deviceAdded(s.name, s);
+        ossia::qt::run_async(
+            this, [this, s = to_settings(m_api, p)] { deviceAdded(s.name, s); });
       };
     }
     return cb;
@@ -96,36 +113,35 @@ class MidiEnumerator : public Device::DeviceEnumerator
   libremidi::observer m_observer;
 
 public:
-  MidiEnumerator()
-      : m_observer{make_callbacks(), libremidi::observer_configuration_for(m_api)}
+  explicit MidiEnumerator(libremidi::observer_configuration& cb)
+      : m_observer{make_callbacks(cb), libremidi::observer_configuration_for(m_api)}
   {
   }
+
   void enumerate(std::function<void(const QString&, const Device::DeviceSettings&)> f)
       const override
   {
-    try
-    {
-      if constexpr(Type == ossia::net::midi::midi_info::Type::Input)
-      {
-        for(auto& port : m_observer.get_input_ports())
-        {
-          auto s = to_settings(m_api, port);
-          f(s.name, s);
-        }
-      }
-      else if constexpr(Type == ossia::net::midi::midi_info::Type::Output)
-      {
-        for(auto& port : m_observer.get_output_ports())
-        {
-          auto s = to_settings(m_api, port);
-          f(s.name, s);
-        }
-      }
-    }
-    catch(std::exception& e)
-    {
-      qDebug() << e.what();
-    }
+  }
+};
+
+class MidiKeyboardEnumerator : public Device::DeviceEnumerator
+{
+public:
+  void enumerate(std::function<void(const QString&, const Device::DeviceSettings&)> f)
+      const override
+  {
+    Device::DeviceSettings set;
+    MIDISpecificSettings specif;
+    set.name = QString::fromStdString("Computer keyboard");
+    set.protocol = MIDIInputProtocolFactory::static_concreteKey();
+    specif.handle = libremidi::port_information{.display_name = "Computer keyboard"};
+
+    specif.io = MIDISpecificSettings::IO::In;
+    specif.api = libremidi::API::KEYBOARD;
+
+    set.deviceSpecificSettings = QVariant::fromValue(specif);
+
+    f("Computer keyboard", set);
   }
 };
 
@@ -144,10 +160,26 @@ QString MIDIInputProtocolFactory::category() const noexcept
   return StandardCategories::hardware;
 }
 
+QUrl MIDIInputProtocolFactory::manual() const noexcept
+{
+  return QUrl("https://ossia.io/score-docs/devices/midiin-device.html");
+}
+
 Device::DeviceEnumerators
 MIDIInputProtocolFactory::getEnumerators(const score::DocumentContext& ctx) const
 {
-  return {{"Inputs", new MidiEnumerator<ossia::net::midi::midi_info::Type::Input>}};
+  libremidi::observer_configuration obs_hw, obs_sw;
+  obs_hw.track_hardware = true;
+  obs_hw.track_virtual = false;
+  obs_sw.track_hardware = false;
+  obs_sw.track_virtual = true;
+  return {
+      {"Hardware inputs",
+       new MidiEnumerator<ossia::net::midi::midi_info::Type::Input>(obs_hw)},
+      {"Software inputs",
+       new MidiEnumerator<ossia::net::midi::midi_info::Type::Input>(obs_sw)},
+      {"Other", new MidiKeyboardEnumerator},
+  };
 }
 
 Device::DeviceInterface* MIDIInputProtocolFactory::makeDevice(
@@ -166,6 +198,7 @@ const Device::DeviceSettings& MIDIInputProtocolFactory::defaultSettings() const
     MIDISpecificSettings specif;
     specif.io = MIDISpecificSettings::IO::In;
     specif.virtualPort = false;
+    specif.api = getCurrentAPI();
     s.deviceSpecificSettings = QVariant::fromValue(specif);
     return s;
   }();
@@ -206,7 +239,9 @@ void MIDIInputProtocolFactory::serializeProtocolSpecificSettings(
 bool MIDIInputProtocolFactory::checkCompatibility(
     const Device::DeviceSettings& a, const Device::DeviceSettings& b) const noexcept
 {
-  return a.name != b.name;
+  // FIXME check if we can open the same device multiple times ?
+  auto specif = a.deviceSpecificSettings.value<MIDISpecificSettings>();
+  return specif.handle != libremidi::port_information{} || specif.virtualPort;
 }
 
 QString MIDIOutputProtocolFactory::prettyName() const noexcept
@@ -219,10 +254,25 @@ QString MIDIOutputProtocolFactory::category() const noexcept
   return StandardCategories::hardware;
 }
 
+QUrl MIDIOutputProtocolFactory::manual() const noexcept
+{
+  return QUrl("https://ossia.io/score-docs/devices/midiout-device.html");
+}
+
 Device::DeviceEnumerators
 MIDIOutputProtocolFactory::getEnumerators(const score::DocumentContext& ctx) const
 {
-  return {{"Outputs", new MidiEnumerator<ossia::net::midi::midi_info::Type::Output>}};
+  libremidi::observer_configuration obs_hw, obs_sw;
+  obs_hw.track_hardware = true;
+  obs_hw.track_virtual = false;
+  obs_sw.track_hardware = false;
+  obs_sw.track_virtual = true;
+  return {
+      {"Hardware outputs",
+       new MidiEnumerator<ossia::net::midi::midi_info::Type::Output>(obs_hw)},
+      {"Software outputs",
+       new MidiEnumerator<ossia::net::midi::midi_info::Type::Output>(obs_sw)},
+  };
 }
 
 Device::DeviceInterface* MIDIOutputProtocolFactory::makeDevice(
@@ -241,6 +291,7 @@ const Device::DeviceSettings& MIDIOutputProtocolFactory::defaultSettings() const
     MIDISpecificSettings specif;
     specif.io = MIDISpecificSettings::IO::Out;
     specif.virtualPort = false;
+    specif.api = getCurrentAPI();
     s.deviceSpecificSettings = QVariant::fromValue(specif);
     return s;
   }();
@@ -281,6 +332,8 @@ void MIDIOutputProtocolFactory::serializeProtocolSpecificSettings(
 bool MIDIOutputProtocolFactory::checkCompatibility(
     const Device::DeviceSettings& a, const Device::DeviceSettings& b) const noexcept
 {
-  return a.name != b.name;
+  // FIXME check if we can open the same device multiple times ?
+  auto specif = a.deviceSpecificSettings.value<MIDISpecificSettings>();
+  return (specif.handle != libremidi::port_information{}) || specif.virtualPort;
 }
 }
