@@ -33,7 +33,6 @@
 #include <wobjectimpl.h>
 
 #include <memory>
-#include <set>
 W_OBJECT_IMPL(vst::Model)
 
 // If a VST has less than this many parameters they will be shown by default.
@@ -75,9 +74,8 @@ EffectProcessFactory_T<vst::Model>::descriptor(QString d) const noexcept
   Process::Descriptor desc;
   auto& app = score::GUIAppContext().applicationPlugin<vst::ApplicationPlugin>();
 
-  auto it = ossia::find_if(app.vst_infos, [this, d](const vst::VSTInfo& vst) {
-    return vst.uniqueID == d.toInt();
-  });
+  auto it = ossia::find_if(
+      app.vst_infos, [d](const vst::VSTInfo& vst) { return vst.uniqueID == d.toInt(); });
   if(it != app.vst_infos.end())
   {
     desc.prettyName = it->displayName;
@@ -175,6 +173,13 @@ bool Model::hasExternalUI() const noexcept
 {
   if(!fx)
     return false;
+
+#if defined(__linux__)
+  static const thread_local bool is_wayland = qApp->platformName() == "wayland";
+  if(is_wayland)
+    return false;
+#endif
+
   return bool(fx->fx->flags & VstAEffectFlags::effFlagsHasEditor);
 }
 
