@@ -135,6 +135,12 @@ QString Model::prettyName() const noexcept
 
 bool Model::hasExternalUI() const noexcept
 {
+#if defined(__linux__)
+  static const thread_local bool is_wayland = qApp->platformName() == "wayland";
+  if(is_wayland)
+    return false;
+#endif
+
   return fx.hasUI;
 }
 
@@ -594,8 +600,9 @@ void Model::loadPreset(const Process::Preset& preset)
 
   if(auto it = obj.FindMember("ProgramIndex"); it != obj.MemberEnd())
   {
-    auto idx = JsonValue{it->value}.toInt();
+    // FIXME
     return;
+    // auto idx = JsonValue{it->value}.toInt();
     //    this->fx.units->getProgramInfo();
   }
   else
